@@ -2,52 +2,41 @@
 The control._experiment_control module of expyriment.
 
 """
-from __future__ import absolute_import, print_function, division
-from builtins import *
-
 
 __author__ = 'Florian Krause <florian@expyriment.org>, \
 Oliver Lindemann <oliver@expyriment.org>'
-__version__ = ''
-__revision__ = ''
-__date__ = ''
 
-import sys
 import os
+import sys
+
 import pygame
-try:
-    import android.mixer as mixer
-except ImportError:
-    import pygame.mixer as mixer
 
-from . import defaults
-from ._miscellaneous import _set_stdout_logging
-from .._internals import get_version, android
-from .. import design, stimuli, misc, _internals
-from ..io import DataFile, EventFile, TextInput, Keyboard, Mouse, \
-                _keyboard, TouchScreenButtonBox
+from .. import _internals, design, misc, stimuli
+from .._internals import get_version
+from ..io import DataFile, EventFile, Keyboard, Mouse, TextInput, TouchScreenButtonBox
 from ..io._screen import Screen
+from ..misc import is_android_running
+from . import defaults
+from ._miscellaneous import _set_stdout_logging, start_audiosystem
 
 
-def start(experiment=None, auto_create_subject_id=None, subject_id=None,
-            skip_ready_screen=False):
+def start(auto_create_subject_id=None, subject_id=None, skip_ready_screen=False):
     """Start an experiment.
 
-    This starts an experiment defined by 'experiment' and asks for the subject
+    This starts the currently initialised experiment and asks for the subject
     number. When the subject number is entered and confirmed by ENTER, a data
     file is created.
     Eventually, "Ready" will be shown on the screen and the method waits for
     ENTER to be pressed.
 
-    After experiment start the following additional properties are available:
+    After the experiment has been started, the following additional properties
+    are available:
 
     * experiment.subject -- the current subject id
     * experiment.data    -- the main data file
 
     Parameters
     ----------
-    experiment : design.Experiment, optional (DEPRECATED)
-        don't use this parameter, it only exists to keep backward compatibility
     auto_create_subject_id : bool, optional
         if True new subject id will be created automatically
     subject_id : integer, optional
@@ -64,11 +53,8 @@ def start(experiment=None, auto_create_subject_id=None, subject_id=None,
 
     """
 
-    if experiment is None:
-        experiment = _internals.active_exp
-    if experiment != _internals.active_exp:
-        raise Exception("Experiment is not the currently initialized " +
-                        "experiment!")
+    experiment = _internals.active_exp
+
     if experiment.is_started:
         raise Exception("Experiment is already started!")
     if subject_id is not None:
@@ -91,7 +77,7 @@ def start(experiment=None, auto_create_subject_id=None, subject_id=None,
         default_number = subject_id
 
     if not auto_create_subject_id:
-        if android is not None:
+        if is_android_running():
             background_stimulus = stimuli.BlankScreen(colour=(0, 0, 0))
             fields = [stimuli.Circle(radius=100, colour=(70, 70, 70),
                                      position=(0, 70), anti_aliasing=10),
@@ -150,7 +136,7 @@ def start(experiment=None, auto_create_subject_id=None, subject_id=None,
                 try:
                     experiment._subject = int(subject_id)
                     break
-                except:
+                except Exception:
                     pass
 
     else:
@@ -168,24 +154,31 @@ def start(experiment=None, auto_create_subject_id=None, subject_id=None,
     for f in experiment.bws_factor_names:
         _permuted_bws_factor_condition = \
             experiment.get_permuted_bws_factor_condition(f)
-        experiment.data.add_subject_info(u"{0} = {1}".format(
+        experiment.data.add_subject_info("{0} = {1}".format(
             f, _permuted_bws_factor_condition))
 
     if experiment.events is not None:
         experiment.events._time_stamp = experiment.data._time_stamp
         experiment.events.rename(experiment.events.standard_file_name)
 
-    number = defaults.initialize_delay - int(experiment.clock.time // 1000)
+    initialise_delay = defaults.initialise_delay
+    if hasattr(defaults, "initialize_delay"):
+        initialise_delay = defaults.initialize_delay
+        print(f"WARNING: Both control.defaults.initialise_delay and "
+              f"control.defaults.initialize_delay are defined. The value of "
+              f"control.defaults.initialize_delay ({initialise_delay}) will "
+              f"be used!")
+
+    number = initialise_delay - int(experiment.clock.time // 1000)
     if number > 0:
-        text = stimuli.TextLine("Initializing, please wait...",
+        text = stimuli.TextLine("Initialising, please wait...",
                                 text_size=int(experiment.text_size * 1.2),
                                 text_colour=(160, 70, 250),
                                 position=(0, 0))
         stimuli._stimulus.Stimulus._id_counter -= 1
         text.present()
         text.present()  # for flipping with double buffer
-        text.present()  # for flipping with tripple buffer
-    default_textline_size = stimuli.TextLine(text="").text_size
+        text.present()  # for flipping with triple buffer
     while number > 0:
         counter = stimuli.TextLine(
             "{num:02d}".format(num=number),
@@ -210,10 +203,10 @@ def start(experiment=None, auto_create_subject_id=None, subject_id=None,
                          text_size=int(experiment.text_size * 1.2),
                          text_colour=misc.constants.C_EXPYRIMENT_ORANGE).present()
         stimuli._stimulus.Stimulus._id_counter -= 1
-        if android is None:
-            experiment.keyboard.wait()
-        else:
+        if is_android_running():
             experiment.mouse.wait_press()
+        else:
+            experiment.keyboard.wait()
     experiment.set_log_level(old_logging)
     experiment._screen.colour = screen_colour
     experiment.log_design_to_event_file()
@@ -221,37 +214,47 @@ def start(experiment=None, auto_create_subject_id=None, subject_id=None,
     return experiment
 
 
-def pause():
+def pause(text="Paused", key=misc.constants.K_RETURN):
     """Pause a running experiment.
 
-    This will show a pause screen and waits for ENTER to be pressed to
-    continue.
+    This will show a pause screen and waits for a key (default=ENTER) to be
+    pressed to continue.
+
+    Parameters
+    ----------
+    text : str, optional
+        the text to be shown at the pause screen (default="Paused")
+
+    key : int, optional
+        the key to be pressed for continuing (default=misc.constants.K_RETURN)
+
 
     """
 
-    if not _internals.active_exp.is_initialized:
-        raise Exception("Experiment is not initialized!")
+    if not _internals.active_exp.is_initialised:
+        raise Exception("Experiment is not initialised!")
     experiment = _internals.active_exp
     experiment._event_file_log("Experiment,paused")
     screen_colour = experiment.screen.colour
     experiment._screen.colour = [0, 0, 0]
     old_logging = experiment.log_level
     experiment.set_log_level(0)
-    if android is not None:
+    if is_android_running():
         position = (0, 200)
     else:
         position = (0, 0)
-    stimuli.TextLine("Paused", position=position,
-                     text_colour=misc.constants.C_EXPYRIMENT_ORANGE,
-                     text_size=int(_internals.active_exp.text_size * 1.2)).present()
+    stimuli.TextLine(
+        text, position=position,
+        text_colour=misc.constants.C_EXPYRIMENT_ORANGE,
+        text_size=int(_internals.active_exp.text_size * 1.2)).present()
     experiment.set_log_level(old_logging)
     experiment._screen.colour = screen_colour
     stimuli._stimulus.Stimulus._id_counter -= 1
     misc.Clock().wait(200)
-    if android is None:
-        experiment.keyboard.wait()
-    else:
+    if is_android_running():
         experiment.mouse.wait_press()
+    else:
+        experiment.keyboard.wait(keys=(key), low_performance=True)
     experiment._event_file_log("Experiment,resumed")
 
 
@@ -272,8 +275,8 @@ def end(goodbye_text=None, goodbye_delay=None, confirmation=False,
         (default = None)
     system_exit : bool, optional
         call Python's sys.exit() method when ending expyriment (default = False)
-    pre_quit_function : function, optional
-        function to be called before quitting Pygame
+    pre_quit_function : function or list of functions, optional
+        function(s) to be called before quitting Pygame
 
     Returns
     -------
@@ -282,12 +285,17 @@ def end(goodbye_text=None, goodbye_delay=None, confirmation=False,
 
     """
 
-    if not _internals.active_exp.is_initialized:
+    if not _internals.active_exp.is_initialised:
         if pre_quit_function is not None:
-            pre_quit_function()
+            if isinstance(pre_quit_function, (tuple, list)):
+                for function in pre_quit_function:
+                    function()
+            else:
+                pre_quit_function()
 
         # Delete open file handles and previously opened fonts
-        import expyriment.stimuli._textline, expyriment.stimuli._textbox
+        import expyriment.stimuli._textbox
+        import expyriment.stimuli._textline
         for text_stim in [stimuli._textline, stimuli._textbox]:
             for f in text_stim.open_filehandles:
                 f.close()
@@ -303,7 +311,7 @@ def end(goodbye_text=None, goodbye_delay=None, confirmation=False,
         experiment._event_file_log("Experiment,paused")
         screen_colour = experiment.screen.colour
         experiment._screen.colour = [0, 0, 0]
-        if android is not None:
+        if is_android_running():
             position = (0, 200)
         else:
             position = (0, 0)
@@ -311,7 +319,8 @@ def end(goodbye_text=None, goodbye_delay=None, confirmation=False,
                          text_colour=misc.constants.C_EXPYRIMENT_ORANGE,
                          text_size=int(_internals.active_exp.text_size * 1.2)).present()
         stimuli._stimulus.Stimulus._id_counter -= 1
-        char = Keyboard().wait_char(["y", "n"], process_control_events=False)
+        char = Keyboard().wait_char(["y", "n"], process_control_events=False,
+                                    low_performance=True)
         if char[0] == "n":
             experiment._screen.colour = screen_colour
             experiment._event_file_log("Experiment,resumed")
@@ -341,17 +350,24 @@ def end(goodbye_text=None, goodbye_delay=None, confirmation=False,
                          text_colour=misc.constants.C_EXPYRIMENT_PURPLE,
                          text_size=int(_internals.active_exp.text_size * 1.2)).present()
         stimuli._stimulus.Stimulus._id_counter -= 1
-    except:
+    except Exception:
         pass
 
     if not fast_quit:
         misc.Clock().wait(goodbye_delay)
-    _internals.active_exp = design.Experiment("None")
+
     if pre_quit_function is not None:
-        pre_quit_function()
+        if isinstance(pre_quit_function, (tuple, list)):
+            for function in pre_quit_function:
+                function()
+        else:
+            pre_quit_function()
+
+    _internals.active_exp = design.Experiment("None")
 
     # Delete open file handles and previously opened fonts
-    import expyriment.stimuli._textline, expyriment.stimuli._textbox
+    import expyriment.stimuli._textbox
+    import expyriment.stimuli._textline
     for text_stim in [expyriment.stimuli._textline, expyriment.stimuli._textbox]:
         for f in text_stim.open_filehandles:
             f.close()
@@ -362,20 +378,19 @@ def end(goodbye_text=None, goodbye_delay=None, confirmation=False,
         sys.exit()
     return True
 
+def initialise(experiment=None):
+    """Initialise an experiment.
 
-def initialize(experiment=None):
-    """Initialize an experiment.
-
-    This initializes an experiment defined by 'experiment' as well as the
+    This initialises an experiment defined by 'experiment' as well as the
     underlying expyriment system. If 'experiment' is None, a new Experiment
     object will be created and returned. Furthermore, a screen, a clock, a
-    keyboard and a event file are created and added to the experiment. The
-    initialization screen is shown for a short delay to ensure that Python
-    is fully initialized and time accurate. Afterwards, "Preparing
+    keyboard and an event file are created and added to the experiment. The
+    initialisation screen is shown for a short delay to ensure that Python
+    is fully initialised and time accurate. Afterwards, "Preparing
     experiment..." is presented on the screen.
 
-    After experiment initialize the following additional properties are
-    available:
+    After the experiment has been initialised, the following additional
+    properties are available:
 
     - experiment.screen   -- the current screen
     - experiment.clock    -- the main clock
@@ -386,12 +401,12 @@ def initialize(experiment=None):
     Parameters
     ----------
     experiment : design.Experiment, optional
-        the experiment to initialize
+        the experiment to initialise
 
     Returns
     -------
     exp : design.Experiment
-        initialized experiment
+        initialised experiment
 
     """
 
@@ -404,7 +419,7 @@ def initialize(experiment=None):
     if misc.is_interactive_mode() and not defaults.window_mode \
         and not hasattr(experiment, "testsuite"):
         print("""
-Python is running in an interactive shell but Expyriment wants to initialize a
+Python is running in an interactive shell but Expyriment wants to initialise a
 fullscreen.""")
         quest = "Do you want to switch to windows mode?"
         ans = input(quest + " (Y/n) ").strip().lower()
@@ -417,25 +432,34 @@ fullscreen.""")
     old_logging = experiment.log_level
     experiment.set_log_level(0)  # switch off for the first screens
 
-    _keyboard.quit_key = defaults.quit_key
-    _keyboard.pause_key = defaults.pause_key
-    _keyboard.end_function = end
-    _keyboard.pause_function = pause
-
-    mixer.pre_init(defaults.audiosystem_sample_rate,
-                   defaults.audiosystem_bit_depth,
-                   defaults.audiosystem_channels,
-                   defaults.audiosystem_buffer_size)
+    if defaults.audiosystem_bit_depth not in (8, -8, 16, -16, 32):
+        message = "Audiosystem only supports bit depth values of " + \
+            "8, -8, 16, -16 and 32."
+        raise RuntimeError(message.format(defaults.audiosystem.bit_depth))
+    if defaults.audiosystem_channels not in (1, 2, 4, 6):
+        message = "Audiosystem only supports channel values of 1, 2, 4 and 6."
+        raise RuntimeError(message.format(defaults.audiosystem.channels))
+    pygame.mixer.pre_init(defaults.audiosystem_sample_rate,
+                          defaults.audiosystem_bit_depth,
+                          defaults.audiosystem_channels,
+                          defaults.audiosystem_buffer_size,
+                          devicename=defaults.audiosystem_device,
+                          allowedchanges=0)
     if defaults.audiosystem_autostart:
-        mixer.init()
-        mixer.init()  # Needed on some systems
+        start_audiosystem()
 
     experiment._clock = misc.Clock()
+    if hasattr(defaults, "open_gl"):
+        raise DeprecationWarning("'expyriment.control.defaults.open_gl' is " +\
+                                 "DEPRECATED! Please use " + \
+                                 "'expyriment.control.defaults.opengl'.")
     experiment._screen = Screen(colour=(0, 0, 0),
-                                open_gl=defaults.open_gl,
+                                opengl=defaults.opengl,
                                 window_mode=defaults.window_mode,
                                 window_size=defaults.window_size,
-                                no_frame=defaults.window_no_frame)
+                                no_frame=defaults.window_no_frame,
+                                display=defaults.display,
+                                display_resolution=defaults.display_resolution)
     # Hack for IDLE: quit pygame and call atexit functions when crashing
     if misc.is_idle_running() and sys.argv[0] != "":
         try:
@@ -454,8 +478,8 @@ fullscreen.""")
             pass
     experiment._data = None
     experiment._subject = None
-    experiment._is_initialized = True  # required before EventFile
-    if old_logging> 0:
+    experiment._is_initialised = True  # required before EventFile
+    if old_logging > 0:
         experiment._events = EventFile(
             additional_suffix=experiment.filename_suffix, time_stamp=True)
         if stdout_logging:
@@ -463,17 +487,21 @@ fullscreen.""")
     else:
         experiment._events = None
     experiment._keyboard = Keyboard()
+    experiment._keyboard.quit_control.setup(quit_key=defaults.quit_key,
+                                            end_function=end)
     experiment._mouse = Mouse(show_cursor=False)
+    # Scale fonts/logo according to default font size
+    scaling = experiment.text_size / 20
     logo = stimuli.Picture(misc.constants.EXPYRIMENT_LOGO_FILE,
                            position=(0, 100))
-    logo.scale((0.7, 0.7))
+    logo.scale(0.6 * scaling)
     text = stimuli.TextLine("Version {0}".format(get_version()),
-                            text_size= experiment.text_size,
+                            text_size=int(20 * scaling),
                             text_colour=misc.constants.C_EXPYRIMENT_PURPLE,
                             background_colour=(0, 0, 0),
-                            position=(0, -5))
-    canvas = stimuli.Canvas((600, 400), colour=(0, 0, 0))
-    canvas2 = stimuli.Canvas((600, 400), colour=(0, 0, 0))
+                            position=(0, -int(25 * scaling ** 2.5)))
+    canvas = stimuli.Canvas((800, 600), colour=(0, 0, 0))
+    canvas2 = stimuli.Canvas((800, 600), colour=(0, 0, 0))
     logo.plot(canvas)
     text.plot(canvas)
     hash_ = misc.get_experiment_secure_hash()
@@ -483,17 +511,26 @@ fullscreen.""")
             txt += ", {0}".format(
                         misc.module_hashes_as_string())
         text2 = stimuli.TextLine(txt,
-            text_size= int(experiment.text_size * 0.7),
+            text_size=int(14 * scaling),
             text_colour=misc.constants.C_EXPYRIMENT_ORANGE,
             background_colour=(0, 0, 0),
-            position=(0, -50))
+            position=(0, -int(75 * scaling ** 1.6)))
         text2.plot(canvas)
     canvas.preload(True)
     canvas._set_surface(canvas._get_surface().convert())
     start = experiment.clock.time
     r = [x for x in range(256) if x % 5 == 0]
     stopped = False
-    if defaults.initialize_delay > 0:
+
+    initialise_delay = defaults.initialise_delay
+    if hasattr(defaults, "initialize_delay"):
+        initialise_delay = defaults.initialize_delay
+        print(f"WARNING: Both control.defaults.initialise_delay and "
+              f"control.defaults.initialize_delay are defined. The value of "
+              f"control.defaults.initialize_delay ({initialise_delay}) will "
+              f"be used!")
+
+    if initialise_delay > 0:
         for x in r:
             canvas._get_surface().set_alpha(x)
             canvas2.clear_surface()
