@@ -7,16 +7,12 @@ experiment.  See also expyriment.design.extras for more design.
 
 __author__ = 'Florian Krause <florian@expyriment.org> \
               Oliver Lindemann <oliver@expyriment.org>'
-__version__ = ''
-__revision__ = ''
-__date__ = ''
 
 
-from . import defaults
-from . import permute
-from . import randomize
-from ._structure import Experiment, Block, Trial
-
 from .. import _internals
+
+from . import defaults, permute, randomise, randomize
+from ._structure import Block, Experiment, Trial
+
 _internals.active_exp = Experiment("None")
 
