@@ -4,41 +4,43 @@ The design._structure module of expyriment.
 This module contains a class implementing the experiment structure.
 
 """
-from __future__ import absolute_import, print_function, division
-from builtins import *
 
 __author__ = 'Florian Krause <florian@expyriment.org>, \
 Oliver Lindemann <oliver@expyriment.org>'
-__version__ = ''
-__revision__ = ''
-__date__ = ''
-
 
 import os
+
 try:
     import locale
 except ImportError:
     locale = None  # Does not exist on Android
-import sys
-from types import FunctionType
 import codecs
 import re
+import sys
+from types import FunctionType
+
 try:
     import csv
 except ImportError:
     from ..misc import _csv_reader_android as csv
+
 from copy import deepcopy
 
-from . import defaults
 from .. import _internals
-from ..misc import constants, Clock, unicode2byte, byte2unicode, py2py3_sort_array
-from .randomize import rand_int, shuffle_list
-from . import permute
+from ..misc import (
+    Clock,
+    bytes_to_unicode,
+    constants,
+    string_sort_array,
+    unicode_to_bytes,
+)
+from . import defaults, permute
+from .randomise import rand_int, shuffle_list
 
 _FACTOR_NOT_EXIST = "The factor '{0}' does not exist!\nUse has_factor(name) to check if a factor is defined."
 _BWS_FACTOR_NOT_EXIST = "The bws-factor '{0}' does not exist!\nUse has_bws_factor(name) to check if a bws-factor is defined."
 
-class Experiment(object):
+class Experiment:
     """A class implementing a basic experiment."""
 
     def __init__(self, name=None, foreground_colour=None,
@@ -95,7 +97,7 @@ class Experiment(object):
         self._block_id_counter = 0
 
         self._is_started = False
-        self._is_initialized = False
+        self._is_initialised = False
         self._keyboard = None
         self._mouse = None
         self._clock = None
@@ -103,7 +105,7 @@ class Experiment(object):
         self._screen = None
         self._data = None
         self._events = None
-        self._log_level = None  # will be set from initialize
+        self._log_level = None  # will be set from initialise
         self._wait_callback_function = None
 
     @property
@@ -203,29 +205,35 @@ class Experiment(object):
         return self._is_started
 
     @property
-    def is_initialized(self):
-        """Getter for is_initialized."""
+    def is_initialised(self):
+        """Getter for is_initialised."""
 
-        return self._is_initialized
+        return self._is_initialised
+
+    @property
+    def is_initialized(self):
+        """Getter for is_initialised."""
+
+        return self._is_initialised
 
     def __str__(self):
-        tmp_str = u"Experiment: {0}\n".format(self.name)
+        tmp_str = "Experiment: {0}\n".format(self.name)
         if len(self.bws_factor_names) <= 0:
-            tmp_str = tmp_str + "no between subject factors\n"
+            tmp_str += "no between subject factors\n"
         else:
-            tmp_str = tmp_str + "between subject factors (permutation type: "
-            if self.bws_factor_randomized:
-                tmp_str = tmp_str + "random)\n"
+            tmp_str += "between subject factors (permutation type: "
+            if self.bws_factor_randomised:
+                tmp_str += "random)\n"
             else:
-                tmp_str = tmp_str + "latin square)\n"
+                tmp_str += "latin square)\n"
             for f in self.bws_factor_names:
                 _bws_factor = \
                     [x if isinstance(x, str) else
                      repr(x) for x in self._bws_factors[f]]
-                tmp_str = tmp_str + u"    {0} = [{1}]\n".format(
-                    f, u", ".join(_bws_factor))
+                tmp_str += "    {0} = [{1}]\n".format(
+                    f, ", ".join(_bws_factor))
         for block in self.blocks:
-            tmp_str = tmp_str + u"{0}\n".format(block.summary)
+            tmp_str += "{0}\n".format(block.summary)
         return tmp_str
 
     @property
@@ -261,7 +269,7 @@ class Experiment(object):
             return
         try:
             variable_names = list(variable_names)
-        except:
+        except Exception:
             variable_names = [variable_names]
         self._data_variable_names.extend(variable_names)
         if self.data is not None:
@@ -290,7 +298,7 @@ class Experiment(object):
         else:
             try:
                 text = list(text)
-            except:
+            except Exception:
                 text = [text]
 
         self._experiment_info.extend(text)
@@ -299,26 +307,48 @@ class Experiment(object):
                 self.data.add_experiment_info(elem)
 
     @property
-    def bws_factor_randomized(self):
-        """Getter for bws_factor_randomized.
+    def bws_factor_randomised(self):
+        """Getter for bws_factor_randomised.
 
         Notes
         -----
-        Is between subject factor randomized? (True/False).
+        Is between subject factor randomised? (True/False).
 
-        If True conditions will be assigned randomized
+        If True conditions will be assigned randomised
         otherwise (default) conditions will be systematically permuted across
         subjects.
 
         """
 
-        return self._bws_factor_randomized
+        return self._bws_factor_randomised
+
+    @bws_factor_randomised.setter
+    def bws_factor_randomised(self, value):
+        """Setter for bws_factor_randomised."""
+
+        self._bws_factor_randomised = value
+
+    @property
+    def bws_factor_randomized(self):
+        """Getter for bws_factor_randomised.
+
+        Notes
+        -----
+        Is between subject factor randomised? (True/False).
+
+        If True conditions will be assigned randomised
+        otherwise (default) conditions will be systematically permuted across
+        subjects.
+
+        """
+
+        return self._bws_factor_randomised
 
     @bws_factor_randomized.setter
     def bws_factor_randomized(self, value):
-        """Setter for bws_factor_randomized."""
+        """Setter for bws_factor_randomised."""
 
-        self._bws_factor_randomized = value
+        self._bws_factor_randomised = value
 
     def add_bws_factor(self, factor_name, conditions):
         """Add a between subject factor.
@@ -345,11 +375,11 @@ class Experiment(object):
 
         try:
             conditions = list(conditions)
-        except:
+        except Exception:
             conditions = [conditions]
         self._bws_factors[factor_name] = conditions
         self._bws_factors_names.append(factor_name)
-        self._randomized_condition_for_subject[factor_name] = {}
+        self._randomised_condition_for_subject[factor_name] = {}
 
     def has_bws_factor(self, factor_name):
         """Checks if a factor is defined.
@@ -386,7 +416,7 @@ class Experiment(object):
 
         try:
             cond = self._bws_factors[factor_name]
-        except:
+        except Exception:
             if return_none_if_not_defined:
                 return None
             else:
@@ -430,14 +460,14 @@ class Experiment(object):
                 return None  # Factor not defined
             else:
                 cond_idx = 0
-                if self.bws_factor_randomized:
+                if self.bws_factor_randomised:
                     try:
-                        cond_idx = self._randomized_condition_for_subject[
+                        cond_idx = self._randomised_condition_for_subject[
                             factor_name][subject_id]
-                    except:  # If not yet randomized for this subject, do it
+                    except Exception:  # If not yet randomised for this subject, do it
                         cond_idx = rand_int(
                             0, len(self._bws_factors[factor_name]) - 1)
-                        self._randomized_condition_for_subject[
+                        self._randomised_condition_for_subject[
                             factor_name][subject_id] = cond_idx
 
                 else:  # Permutation
@@ -462,8 +492,8 @@ class Experiment(object):
         # Can't use dict_keys, because dicts don't keep the order
         self._bws_factors_names = []
 
-        self._randomized_condition_for_subject = {}
-        self.bws_factor_randomized = False
+        self._randomised_condition_for_subject = {}
+        self.bws_factor_randomised = False
 
     @property
     def bws_factor_names(self):
@@ -500,12 +530,12 @@ class Experiment(object):
         - 2 intensive logging; logs much more; please use this only for
           debugging proposes
 
-        In most cases, it should be avoided to switch of logging (loglevel=0).
+        In most cases, it should be avoided to switch off logging (loglevel=0).
         If log files become to big due to certain repetitive events, it is
         suggested to switch of the logging of individual stimuli or IO event.
         (see the method `.set_logging()` of this objects)
 
-        The logging of events can be also changed before initialize via the
+        The logging of events can be also changed before initialise via the
         default value `expyriment.control.defaults.event_logging`.
 
         """
@@ -640,29 +670,72 @@ class Experiment(object):
         else:
             return False
 
-    def shuffle_blocks(self, max_repetitions=None, n_segments=None):
+    def shuffle_blocks(self, method=0, max_repetitions=None, n_segments=None):
         """Shuffle all blocks.
 
+        The function returns False if no randomisation could be found that
+        fulfills the max immediate block repetition criterion. The different
+        type of blocks are only defined by the factors. Shuffle does not
+        take into account the added trials.
+
+        The following randomisation methods are defined:
+
+                0 = total randomisation of block order (default)
+
+                1 = randomisation within small miniblocks. Each miniblock
+                contains one block of each type (only defined by factors!);
+                in other words, copies of one block type are always in
+                different miniblocks
+
         Parameters
         ----------
+        method : int, optional
+            method of block randomisation (default=0)
         max_repetitions : int, optional
-            see documentation of `randomize.shuffle_list`, default = None
+            see documentation of `randomise.shuffle_list` (default = None)
         n_segments : int, optional
-            see documentation of `randomize.shuffle_list`, default = None
+            this parameter will be only considered for randomisation method 0;
+            see documentation of `randomise.shuffle_list` (default = None)
 
         Returns
         -------
-        success : bool
-            returns if randomization was successful and fulfilled the specified
+        succeeded : bool
+            returns if randomisation was successful and fulfilled the specified
             constrains (see max_repetitions)
 
-        See Also
-        --------
-        expyriment.design.randomize.shuffle_list
-
         """
 
-        return shuffle_list(self._blocks)
+        if method == 1: # make segments
+            tmp = self._blocks
+            self._blocks = []
+            types_occurred = []
+            cnt = 0
+            n_segments = 1
+            while len(tmp) > 0:
+                is_new = True
+                tr_type = tmp[cnt].factors_as_text
+                for occ in types_occurred:
+                    if tr_type == occ:
+                        is_new = False
+                        break
+                if is_new:
+                    self._blocks.append(tmp.pop(cnt))
+                    types_occurred.append(tr_type)
+                    cnt = 0
+                else:
+                    cnt += 1
+                    if cnt >= len(tmp):
+                        types_occurred = []
+                        cnt = 0
+                        if len(tmp)>0:
+                            n_segments += 1
+
+        rtn = shuffle_list(self._blocks, max_repetitions=max_repetitions,
+                           n_segments=n_segments)
+        if rtn is False:
+            print("Warning: Could not find an appropriate block " + \
+                  "randomisation!")
+        return rtn
 
     def permute_blocks(self, permutation_type, factor_names=None,
                        subject_id=None):
@@ -692,7 +765,7 @@ a subject number needs to be defined for the permutation.")
                 subject_id = self.subject
 
         if not permute.is_permutation_type(permutation_type):
-                raise AttributeError("{0} is a unknown permutation \
+                raise AttributeError("{0} is an unknown permutation \
 type".format(permutation_type))
         if factor_names is None:
             factor_names = self.block_list_factor_names
@@ -711,12 +784,10 @@ type".format(permutation_type))
                 all_factor_combi.append(combi)
 
         # Get the permutation
-        if permutation_type == constants.P_BALANCED_LATIN_SQUARE:
-            permutation = permute.balanced_latin_square(all_factor_combi)
-            idx = (subject_id - 1) % len(permutation)
-            permutation = permutation[idx]
-        elif permutation_type == constants.P_CYCLED_LATIN_SQUARE:
-            permutation = permute.cycled_latin_square(all_factor_combi)
+        if permutation_type in [constants.P_BALANCED_LATIN_SQUARE,
+                                constants.P_CYCLED_LATIN_SQUARE]:
+            permutation = permute.latin_square(all_factor_combi,
+                                               permutation_type=permutation_type)
             idx = (subject_id - 1) % len(permutation)
             permutation = permutation[idx]
         else:
@@ -739,8 +810,7 @@ type".format(permutation_type))
         """Sort the blocks according to their indices from low to high."""
 
         blocks_new = []
-        id_list = [x.id for x in self._blocks]
-        id_list.sort()
+        id_list = sorted(x.id for x in self._blocks)
         for id in id_list:
             position = [i for i, x in enumerate(self._blocks)
                         if x.id == id][0]
@@ -757,8 +827,8 @@ type".format(permutation_type))
 
         Returns
         -------
-        pos: int
-         positions as a list or None if not in block list.
+        pos: list of int
+            positions as a list or None if not in block list.
 
         """
 
@@ -777,7 +847,7 @@ type".format(permutation_type))
         factors = []
         for bl in self.blocks:
             factors.extend(bl.trial_factor_names)
-        return list(set(factors))
+        return sorted(set(factors))
 
     @property
     def block_list_factor_names(self):
@@ -790,50 +860,52 @@ type".format(permutation_type))
         factors = []
         for bl in self.blocks:
             factors.extend(bl.factor_names)
-        return list(set(factors))
+        return sorted(set(factors))
 
     @property
     def design_as_text(self):
-        """Getter for desing_as_text.
+        """Getter for design_as_text.
 
         Trial list as csv table.
 
         """
 
-        rtn = u"#exp: {0}\n".format(self.name)
+        rtn = "#exp: {0}\n".format(self.name)
         if len(self.experiment_info) > 0:
             for txt in self.experiment_info:
-                rtn += u"#xpi: {0}\n".format(txt)
+                rtn = f"{rtn}#xpi: {txt}\n"
         if len(self.bws_factor_names) > 0:
             for factor_name in self.bws_factor_names:
-                rtn += u"#bws: {0}=".format(factor_name)
+                rtn = f"{rtn}#bws: {factor_name}="
                 for txt in self.get_bws_factor(factor_name):
-                    rtn += u"{0},".format(txt)
+                    rtn = f"{rtn}{txt},"
                 rtn = rtn[:-1] + "\n"  # delete last comma
-            rtn += u"#bws-rand: {0}\n".format(int(self.bws_factor_randomized))
+            rtn += "#bws-rand: {0}\n".format(int(self.bws_factor_randomised))
         if len(self.data_variable_names) > 0:
-            rtn += "#dvn: "
+            rtn = f"{rtn}#dvn: "
             for txt in self.data_variable_names:
-                rtn += u"{0},".format(txt)
+                rtn = f"{rtn}{txt},"
             rtn = rtn[:-1] + "\n"
 
-        rtn += "block_cnt,block_id"
+        rtn = f"{rtn}block_cnt,block_id"
         bl_factors = self.block_list_factor_names
         factors = self.trial_factor_names
         for f in bl_factors:
-            rtn += u",block_{0}".format(f)
-        rtn += ",trial_cnt,trial_id"
+            rtn = f"{rtn},block_{f}"
+        rtn = f"{rtn},trial_cnt,trial_id"
         for f in factors:
-            rtn += u",{0}".format(f)
+            rtn = f"{rtn},{f}"
 
         for bl_cnt, bl in enumerate(self.blocks):
             for tr_cnt, tr in enumerate(bl.trials):
-                rtn += u"\n{0},{1}".format(bl_cnt, bl.id)
+                rtn = f"{rtn}\n{bl_cnt},{bl.id}"
                 for f in bl_factors:
-                    rtn += u",{0}".format(bl.get_factor(f, return_none_if_not_defined=True))
-                rtn += u",{0},{1}".format(tr_cnt, tr.id)
+                    factor = bl.get_factor(f, return_none_if_not_defined=True)
+                    rtn = f"{rtn},{factor}"
+                rtn = f"{rtn},{tr_cnt},{tr.id}"
                 for f in factors:
-                    rtn += u",{0}".format(tr.get_factor(f, return_none_if_not_defined=True))
+                    factor = tr.get_factor(f, return_none_if_not_defined=True)
+                    rtn = f"{rtn},{factor}"
 
         return rtn
 
@@ -858,10 +930,10 @@ type".format(permutation_type))
         with open(filename, 'wb') as f:
             try:
                 locale_enc = locale.getdefaultlocale()[1]
-            except:
+            except Exception:
                 locale_enc = "UTF-8"
             header = "# -*- coding: {0} -*-\n".format(locale_enc)
-            f.write(unicode2byte(header + self.design_as_text))
+            f.write(unicode_to_bytes(header + self.design_as_text))
 
     def load_design(self, filename, encoding=None):
         """Load the design from a csv file containing list of trials.
@@ -891,10 +963,10 @@ type".format(permutation_type))
         if encoding is None:
             with open(filename, 'r') as fl:
                 first_line = fl.readline()
-                encoding = re.findall("coding[:=]\s*([-\w.]+)", first_line)
+                encoding = re.findall(r"coding[:=]\s*([-\w.]+)", first_line)
                 if encoding == []:
                     second_line = fl.readline()
-                    encoding = re.findall("coding[:=]\s*([-\w.]+)",
+                    encoding = re.findall(r"coding[:=]\s*([-\w.]+)",
                                           second_line)
                     if encoding == []:
                         encoding = [None]
@@ -902,7 +974,7 @@ type".format(permutation_type))
             encoding = [encoding]
         with codecs.open(filename, 'rb', encoding[0], errors='replace') as fl:
             for ln in fl:
-                ln = byte2unicode(ln)
+                ln = bytes_to_unicode(ln)
                 if ln[0] == "#":
                     if ln.startswith("#exp:"):
                         self._name = ln[6:].strip()
@@ -912,7 +984,7 @@ type".format(permutation_type))
                         for tmp in ln[6:].split(","):
                             self.add_data_variable_names(tmp.strip())
                     elif ln.startswith("#bws-rand:"):
-                        self.bws_factor_randomized = (ln[11] == "1")
+                        self.bws_factor_randomised = (ln[11] == "1")
                     elif ln.startswith("#bws:"):
                         tmp = ln[6:].split("=")
                         print(tmp[1].strip().split(","))
@@ -939,7 +1011,7 @@ type".format(permutation_type))
                             message = "Can't read design file. " + \
                                 "The file '{0}' ".format(filename) + \
                                 "does not contain an Expyriment trial list."
-                            raise IOError(message)
+                            raise OSError(message)
                     else:
                         block_cnt = None
                         trial_cnt = None
@@ -950,12 +1022,12 @@ type".format(permutation_type))
                             if val.find(".") >= 0:
                                 try:
                                     val = float(val)
-                                except:
+                                except Exception:
                                     pass
                             else:
                                 try:
                                     val = int(val)
-                                except:
+                                except Exception:
                                     pass
 
                             # set value to block or trial
@@ -987,7 +1059,7 @@ type".format(permutation_type))
     def _event_file_log(self, log_text, log_level=1, log_event_tag=None):
         # log_level 1 = default, 2 = extensive, 0 or False = off
         """ Helper function to log event in the global experiment event file"""
-        if self.is_initialized and\
+        if self.is_initialised and\
                 self._log_level > 0 and\
                 self._log_level >= log_level and \
                 self.events is not None:
@@ -995,7 +1067,7 @@ type".format(permutation_type))
 
     def _event_file_warn(self, warning, log_level=1):
         """ Helper function to log event in the global experiment event file"""
-        if self.is_initialized and\
+        if self.is_initialised and\
                 self._log_level > 0 and\
                 self._log_level >= log_level and \
                 self.events is not None:
@@ -1004,7 +1076,7 @@ type".format(permutation_type))
     def log_design_to_event_file(self, additional_comment=""):
         """Log the design (as comment) to the current main event file.
 
-        If no experiment is initialized or no event file exists the function
+        If no experiment is initialised or no event file exists the function
         will not do anything. This function will be automatically called after
         an experiment has been started.
 
@@ -1019,12 +1091,12 @@ type".format(permutation_type))
 
         """
 
-        if self.is_initialized and self.events is not None:
-            self.events.log(u"design,log,{0}".format(additional_comment))
+        if self.is_initialised and self.events is not None:
+            self.events.log("design,log,{0}".format(additional_comment))
             for ln in self.design_as_text.splitlines():
                 self.events.write_comment(
-                    u"design: {0}".format(ln).replace(":#", "-"))
-            self.events.log(u"design,logged,{0}".format(
+                    "design: {0}".format(ln).replace(":#", "-"))
+            self.events.log("design,logged,{0}".format(
                 additional_comment))
 
     def register_wait_callback_function(self, function):
@@ -1107,7 +1179,7 @@ type".format(permutation_type))
         return (self._wait_callback_function is not None)
 
 
-class Block(object):
+class Block:
     """A class implementing an experimental block."""
 
     _trial_cnt_variable_name = "trial_cnt"  # variable names for csv in/output
@@ -1167,26 +1239,26 @@ class Block(object):
             name = ""
         else:
             name = self.name
-        rtn = u"""Block {0}: {1}
+        rtn = """Block {0}: {1}
     block factors: {2}
     n trials: {3}""".format(self.id, name,
                             self.factors_as_text,
                             len(self.trials))
 
         if include_trial_IDs:
-            rtn = rtn + u"""
+            rtn += """
     trial IDs = {0}""".format([t.id for t in self.trials])
-        rtn = rtn + u"""
+        rtn += """
     trial factors: """
         for f in self.trial_factor_names:
             val = []
             for tf in self.get_trial_factor_values(f):
                 if tf not in val:
                     val.append(tf)
-            py2py3_sort_array(val)
+            string_sort_array(val)
             val = [repr(x) if not isinstance(x, str) \
                    else x for x in val]
-            rtn = rtn + u"{0} = [{1}]\n                   ".format(
+            rtn += "{0} = [{1}]\n                   ".format(
                 f, ", ".join(val))
 
         return rtn
@@ -1202,7 +1274,7 @@ class Block(object):
         all_factors = ""
         for f in self.factor_names:
             all_factors = all_factors + \
-                u"{0} = {1}\n                   ".format(
+                "{0} = {1}\n                   ".format(
                     f, self.get_factor(f, return_none_if_not_defined=True))
         all_factors = all_factors.rstrip()
         if len(all_factors) >= 1 and all_factors[-1] == ",":
@@ -1263,7 +1335,7 @@ class Block(object):
 
         try:
             rtn = self._factors[name]
-        except:
+        except Exception:
             if return_none_if_not_defined:
                 return None
             else:
@@ -1355,7 +1427,7 @@ class Block(object):
         log_txt = "Block,trial added,{0}, {1}".format(self.name,
                                                       self._trials[-1]._id)
         if random_position:
-            log_txt = log_txt + ", random position"
+            log_txt += ", random position"
         _internals.active_exp._event_file_log(log_txt, 2)
 
     def remove_trial(self, position):
@@ -1443,11 +1515,11 @@ class Block(object):
                                self._trial_id_variable_name)
         factors = self.trial_factor_names
         for f in factors:
-            rtn = rtn + ",{0}".format(f)
+            rtn += ",{0}".format(f)
         for cnt, tr in enumerate(self.trials):
-            rtn = rtn + "\n{0},{1}".format(cnt, tr.id)
+            rtn += "\n{0},{1}".format(cnt, tr.id)
             for f in factors:
-                rtn = rtn + ",{0}".format(tr.get_factor(f, return_none_if_not_defined=True))
+                rtn += ",{0}".format(tr.get_factor(f, return_none_if_not_defined=True))
         return rtn
 
     def save_design(self, filename):
@@ -1464,10 +1536,10 @@ class Block(object):
         with open(filename, 'wb') as f:
             try:
                 locale_enc = locale.getdefaultlocale()[1]
-            except:
+            except Exception:
                 locale_enc = "UTF-8"
             header = "# -*- coding: {0} -*-\n".format(locale_enc)
-            f.write(unicode2byte(header + self.design_as_text))
+            f.write(unicode_to_bytes(header + self.design_as_text))
 
     def read_design(self, filename):
         """Reads a list of trials from a csv file and clears the old block
@@ -1531,16 +1603,14 @@ class Block(object):
         if encoding is None:
             with open(filename, 'r') as fl:
                 first_line = fl.readline()
-                encoding = re.findall("coding[:=]\s*([-\w.]+)", first_line)
+                encoding = re.findall(r"coding[:=]\s*([-\w.]+)", first_line)
                 if encoding == []:
                     second_line = fl.readline()
-                    encoding = re.findall("coding[:=]\s*([-\w.]+)",
+                    encoding = re.findall(r"coding[:=]\s*([-\w.]+)",
                                           second_line)
                     if encoding == []:
-                        if _internals.PYTHON3:
-                            encoding = ['utf-8']
-                        else:
-                            encoding = [None]
+                        encoding = ['utf-8']
+
         else:
             encoding = [encoding]
 
@@ -1548,12 +1618,12 @@ class Block(object):
             reader = csv.reader(f)
             for r_cnt, row in enumerate(reader):
                 if r_cnt == 0:
-                    factor_names = [byte2unicode(x) for x in row]
+                    factor_names = [bytes_to_unicode(x) for x in row]
                 else:
                     trial = Trial()
                     for c_cnt in range(0, len(row)):
-                        trial.set_factor(byte2unicode(factor_names[c_cnt]),
-                                         byte2unicode(row[c_cnt]))
+                        trial.set_factor(bytes_to_unicode(factor_names[c_cnt]),
+                                         bytes_to_unicode(row[c_cnt]))
                     self.add_trial(trial)
 
 
@@ -1654,20 +1724,19 @@ class Block(object):
                 cnt = 0
         return max_reps
 
-    def shuffle_trials(self, method=0, max_repetitions=None,
-                       n_segments=None):
+    def shuffle_trials(self, method=0, max_repetitions=None, n_segments=None):
         """Shuffle all trials.
 
-        The function returns False if no randomization could be found that
+        The function returns False if no randomisation could be found that
         fulfills the max immediate trial repetition criterion. The different
         type of trials are only defined by the factors. Shuffle does not
         take into account the added stimuli.
 
-        The following randomization methods are defined:
+        The following randomisation methods are defined:
 
-                0 = total randomization of trial order (default)
+                0 = total randomisation of trial order (default)
 
-                1 = randomization within small miniblocks. Each miniblock
+                1 = randomisation within small miniblocks. Each miniblock
                 contains one trial of each type (only defined by factors!);
                 in other words, copies of one trial type are always in
                 different miniblocks
@@ -1675,58 +1744,58 @@ class Block(object):
         Parameters
         ----------
         method : int, optional
-            method of trial randomization (default=0)
+            method of trial randomisation (default=0)
         max_repetitions : int, optional
-            see documentation of `randomize.shuffle_list` (default = None)
+            see documentation of `randomise.shuffle_list` (default = None)
         n_segments : int, optional
-            this parameter will be only considered for randomization method 0;
-            see documentation of `randomize.shuffle_list` (default = None)
+            this parameter will be only considered for randomisation method 0;
+            see documentation of `randomise.shuffle_list` (default = None)
 
         Returns
         -------
         succeeded : bool
+            returns if randomisation was successful and fulfilled the specified
+            constrains (see max_repetitions)
 
         """
 
         if method == 1: # make segments
             tmp = self._trials
             self._trials = []
-            types_occured = []
+            types_occurred = []
             cnt = 0
             n_segments = 1
             while len(tmp) > 0:
                 is_new = True
                 tr_type = tmp[cnt].factors_as_text
-                for occ in types_occured:
+                for occ in types_occurred:
                     if tr_type == occ:
                         is_new = False
                         break
                 if is_new:
                     self._trials.append(tmp.pop(cnt))
-                    types_occured.append(tr_type)
+                    types_occurred.append(tr_type)
                     cnt = 0
                 else:
                     cnt = cnt + 1
                     if cnt >= len(tmp):
-                        types_occured = []
+                        types_occurred = []
                         cnt = 0
                         if len(tmp)>0:
                             n_segments += 1
 
-        rtn = shuffle_list(self._trials,
-                                   max_repetitions=max_repetitions,
-                                   n_segments=n_segments)
-        if rtn == False:
+        rtn = shuffle_list(self._trials, max_repetitions=max_repetitions,
+                           n_segments=n_segments)
+        if rtn is False:
             print("Warning: Could not find an appropriate trial " + \
-                          "randomization!")
+                  "randomisation!")
         return rtn
 
     def sort_trials(self):
         """Sort the trials according to their indices from low to high."""
 
         trials_new = []
-        id_list = [x.id for x in self._trials]
-        id_list.sort()
+        id_list = sorted(x.id for x in self._trials)
         for _id in id_list:
             position = [i for i, x in enumerate(self._trials)
                         if x.id == _id][0]
@@ -1760,14 +1829,14 @@ class Block(object):
         for trial in self._trials:
             owntrials.append(trial)
             triallist.append(trial.copy())
-        self._trials = None
+        self._trials = []
         rtn = deepcopy(self)
         self._trials = owntrials
         rtn._trials = triallist
         return rtn
 
 
-class Trial(object):
+class Trial:
     """A class implementing an experimental trial."""
 
     def __init__(self):
@@ -1851,7 +1920,7 @@ class Trial(object):
 
         try:
             rtn = self._factors[name]
-        except:
+        except Exception:
             if return_none_if_not_defined:
                 return None
             else:
@@ -1880,7 +1949,7 @@ class Trial(object):
         """Return all factor names and values as csv string line"""
         all_factors = ""
         for f in self.factor_names:
-            all_factors = all_factors + u"{0}={1}, ".format(
+            all_factors += "{0}={1}, ".format(
                 f, str(self.get_factor(f, return_none_if_not_defined=True)))
         return all_factors
 
@@ -1983,19 +2052,19 @@ class Trial(object):
         Parameters
         ----------
         max_repetitions : int, optional
-            see documentation of `randomize.shuffle_list`, default = None
+            see documentation of `randomise.shuffle_list`, default = None
         n_segments : int, optional
-            see documentation of `randomize.shuffle_list`, default = None
+            see documentation of `randomise.shuffle_list`, default = None
 
         Returns
         -------
         success : bool
-            randomization was successful and fulfilled the specified
+            randomisation was successful and fulfilled the specified
             constrains (see max_repetitions)
 
         See Also
         ----------
-        expyriment.design.randomize.shuffle_list
+        expyriment.design.randomise.shuffle_list
 
         """
 
@@ -2007,8 +2076,7 @@ class Trial(object):
         """Sort the stimuli according to their IDs from low to high."""
 
         stimuli_new = []
-        id_list = [x.id for x in self._stimuli]
-        id_list.sort()
+        id_list = sorted(x.id for x in self._stimuli)
         for _id in id_list:
             position = [i for i, x in enumerate(self._stimuli)
                         if x.id == _id][0]
@@ -2037,10 +2105,8 @@ class Trial(object):
     def copy(self):
         """Return a copy of the trial."""
 
-        stimlist = []
-        for stim in self._stimuli:
-            stimlist.append(stim)
-        self._stimuli = None
+        stimlist = self._stimuli.copy()
+        self._stimuli = []
         rtn = deepcopy(self)
         self._stimuli = rtn._stimuli = stimlist
         return rtn
