@@ -8,9 +8,6 @@ function of all classes in this package.
 
 __author__ = 'Florian Krause <florian@expyriment.org, \
 Oliver Lindemann <oliver@expyriment.org>'
-__version__ = ''
-__revision__ = ''
-__date__ = ''
 
 
 # Experiment
@@ -20,7 +17,7 @@ experiment_name = None  # Set None if experiment default name should be the
 
 experiment_background_colour = (0, 0, 0)
 experiment_foreground_colour = (150, 150, 150)
-#experiment_text_font = _str2unicode(_os.path.abspath(
+#experiment_text_font = _str_to_unicode(_os.path.abspath(
 #    _os.path.join(_os.path.dirname(__file__),
 #                  "..", "_fonts", "FreeSans.ttf")))
 experiment_text_font = "FreeSans"
