@@ -2,17 +2,14 @@
 
 Notes
 -----
-    To us the extras module you have to import it manually by calling:
-    `import expyriment.design.extras`
+To us the extras module you have to import it manually by calling:
+`import expyriment.design.extras`
 
 
 """
 
 __author__ = 'Florian Krause <florian@expyriment.org> \
 Oliver Lindemann <oliver@expyriment.org>'
-__version__ = ''
-__revision__ = ''
-__date__ = ''
 
 from ... import _internals
 
@@ -27,5 +24,5 @@ for name, code in _internals.import_plugins_code("design").items():
 
 try:
     del (name, code)
-except:
-    pass
\ No newline at end of file
+except Exception:
+    pass
