@@ -4,15 +4,9 @@ An event button box.
 This module contains a class implementing an event button box.
 
 """
-from __future__ import absolute_import, print_function, division
-from builtins import *
 
 __author__ = 'Florian Krause <florian@expyriment.org>, \
 Oliver Lindemann <oliver@expyriment.org>'
-__version__ = ''
-__revision__ = ''
-__date__ = ''
-
 
 from ._streamingbuttonbox import StreamingButtonBox
 
