@@ -4,19 +4,14 @@ File input and output.
 This module contains classes implementing file input and output.
 
 """
-from __future__ import absolute_import, print_function, division
-from builtins import *
 
 __author__ = 'Florian Krause <florian@expyriment.org>, \
 Oliver Lindemann <oliver@expyriment.org>'
-__version__ = ''
-__revision__ = ''
-__date__ = ''
-
 
 
 import atexit
 import os
+
 try:
     import locale
 except ImportError:
@@ -24,18 +19,22 @@ except ImportError:
 import codecs
 import re
 import sys
-import uuid
 import time
-from time import strftime
-from platform import uname
+import uuid
 from itertools import combinations
+from platform import uname
+from time import strftime
 
-from . import defaults
-from .. import _internals
-from .. import misc
+from .. import _internals, misc
+from ..misc import (
+    bytes_to_unicode,
+    get_experiment_secure_hash,
+    module_hashes_as_string,
+    statistics,
+    unicode_to_bytes,
+)
 from ..misc._timer import get_time
-from ..misc import unicode2byte, byte2unicode, get_experiment_secure_hash, module_hashes_as_string
-from ..misc import statistics
+from . import defaults
 from ._input_output import Input, Output
 
 
@@ -62,16 +61,16 @@ class InputFile(Input):
         self._current_line = 1
         self._lines = []
         if not(os.path.isfile(self._filename)):
-            raise IOError(u"The input file '{0}' does not exist.".format(
+            raise OSError("The input file '{0}' does not exist.".format(
                 self._filename))
 
         if encoding is None:
             with open(filename, 'r') as fl:
                 first_line = fl.readline()
-                encoding = re.findall("coding[:=]\s*([-\w.]+)", first_line)
+                encoding = re.findall(r"coding[:=]\s*([-\w.]+)", first_line)
                 if encoding == []:
                     second_line = fl.readline()
-                    encoding = re.findall("coding[:=]\s*([-\w.]+)",
+                    encoding = re.findall(r"coding[:=]\s*([-\w.]+)",
                                           second_line)
                     if encoding == []:
                         encoding = [None]
@@ -80,7 +79,7 @@ class InputFile(Input):
         with codecs.open(self._filename, 'rb', encoding[0],
                          errors='replace') as f:
             for line in f:
-                self._lines.append(byte2unicode(line).rstrip('\r\n'))
+                self._lines.append(bytes_to_unicode(line).rstrip('\r\n'))
 
     @property
     def filename(self):
@@ -183,12 +182,12 @@ class OutputFile(Output):
         fl.close()
         try:
             locale_enc = locale.getdefaultlocale()[1]
-        except:
+        except Exception:
             locale_enc = "UTF-8"
         self.write_comment("Expyriment {0}, {1}-file, coding: {2}".format(
             _internals.get_version(), self._suffix,
             locale_enc))
-        if _internals.active_exp.is_initialized:
+        if _internals.active_exp.is_initialised:
             self.write_comment("date: {0}".format(time.strftime(
                                "%a %b %d %Y %H:%M:%S",
                                _internals.active_exp.clock.init_localtime)))
@@ -228,10 +227,11 @@ class OutputFile(Output):
 
         rtn = os.path.split(sys.argv[0])[1].replace(".py", "")
         if _internals.active_exp.is_started:
-            rtn = rtn + '_' + repr(_internals.active_exp.subject).zfill(2)
+            rtn = f"{rtn}_{_internals.active_exp.subject!r:0>3}"
         if self._time_stamp:
-            rtn = rtn + '_' + strftime(
+            d = strftime(
                 "%Y%m%d%H%M", _internals.active_exp.clock.init_localtime)
+            rtn = f"{rtn}_{d}"
         return rtn + self.suffix
 
     def save(self):
@@ -239,7 +239,7 @@ class OutputFile(Output):
 
         start = get_time()
         if self._buffer != []:
-            buffer = [unicode2byte(x) for x in self._buffer]
+            buffer = [unicode_to_bytes(x) for x in self._buffer]
             with open(self._fullpath, 'ab') as f:
                 f.write(b"".join(buffer))
             self._buffer = []
@@ -257,7 +257,7 @@ class OutputFile(Output):
 
         # NOTE: Do not print here recursion due to std_out logging
 
-        if not _internals.is_base_string(content):
+        if not isinstance(content, (str, bytes)):
             content = str(content)
         self._buffer.append(content)
 
@@ -309,6 +309,14 @@ class OutputFile(Output):
         """Renames the output file."""
         self.save()
         new_fullpath = self.directory + "{0}{1}".format(os.path.sep, new_filename)
+        if os.path.isfile(new_fullpath):
+            cnt = 1
+            while True:
+                old = new_fullpath[:-4] + f"_bak{cnt}" + new_fullpath[-4:]
+                cnt += 1
+                if not os.path.isfile(old):
+                    os.rename(new_fullpath, old)
+                    break
         os.rename(self._fullpath, new_fullpath)
         self._filename = new_filename
         self._fullpath = new_fullpath
@@ -328,8 +336,7 @@ class OutputFile(Output):
             mainfile_name = os.path.split(sys.argv[0])[1].replace(".py", "")
             for filename in file_list:
                 if filename.startswith(mainfile_name) and \
-                        (filename.endswith(DataFile._file_suffix) or
-                         filename.endswith(EventFile._file_suffix)):
+                        filename.endswith((DataFile._file_suffix, EventFile._file_suffix)):
                     tmp = filename.replace(mainfile_name, "")\
                                   .replace(DataFile._file_suffix, "")\
                                   .replace(EventFile._file_suffix, "")
@@ -340,7 +347,7 @@ class OutputFile(Output):
                             # large numbers are probably timestamps (normally the current event has no subject id
                             # yet and only a timestamp)
                             subject_number = num + 1
-                    except:
+                    except Exception:
                         pass
         return subject_number
 
@@ -370,7 +377,7 @@ class DataFile(OutputFile):
 
         """
 
-        if _internals.active_exp.is_initialized:
+        if _internals.active_exp.is_initialised:
             self._subject = _internals.active_exp.subject
         else:
             self._subject = None
@@ -426,17 +433,17 @@ class DataFile(OutputFile):
             line = ""
             for counter, elem in enumerate(data):
                 if counter > 0:
-                    line = line + self.delimiter
-                if not _internals.is_base_string(elem):
+                    line += self.delimiter
+                if not isinstance(elem, (str, bytes)):
                     elem = str(elem)
-                if '"' in byte2unicode(elem):
-                    elem = byte2unicode(elem).replace('"', '""')
-                if ',' in byte2unicode(elem):
-                    elem = '"{0}"'.format(byte2unicode(elem))
-                line = line + elem
+                if '"' in bytes_to_unicode(elem):
+                    elem = bytes_to_unicode(elem).replace('"', '""')
+                if ',' in bytes_to_unicode(elem):
+                    elem = '"{0}"'.format(bytes_to_unicode(elem))
+                line += elem
             self.write_line(line)
         else:
-            if not _internals.is_base_string(data):
+            if not isinstance(data, (str, bytes)):
                 data = str(data)
             self.write_line(data)
 
@@ -460,7 +467,7 @@ class DataFile(OutputFile):
 
         """
 
-        self._subject_info.append(u"{0}s {1}{2}".format(
+        self._subject_info.append("{0}s {1}{2}".format(
             self.comment_char, text, defaults.outputfile_eol))
 
     def add_experiment_info(self, text):
@@ -478,7 +485,7 @@ class DataFile(OutputFile):
         """
 
         for line in text.splitlines():
-            self._experiment_info.append(u"{0}e {1}{2}".format(
+            self._experiment_info.append("{0}e {1}{2}".format(
                 self.comment_char, line, defaults.outputfile_eol))
 
     @property
@@ -518,7 +525,7 @@ class DataFile(OutputFile):
             return
         try:
             variable_names = list(variable_names)
-        except:
+        except Exception:
             variable_names = [variable_names]
         self._variable_names.extend(variable_names)
         self._variable_names_changed = True
@@ -554,21 +561,21 @@ class DataFile(OutputFile):
                 else:
                     if section == "e":  # Previous line was last #e
                         if len(self._experiment_info) > 0:
-                            fl.write(unicode2byte("".join(self._experiment_info)))
+                            fl.write(unicode_to_bytes("".join(self._experiment_info)))
                             self._experiment_info = []
                         section = None
                     elif section == "s":  # Previous line was last #s
                         if len(self._subject_info) > 0:
-                            fl.write(unicode2byte("".join(self._subject_info)))
+                            fl.write(unicode_to_bytes("".join(self._subject_info)))
                             self._subject_info = []
                         section = None
 
                         # Re-write variable names after #s-section
-                        fl.write(unicode2byte(
+                        fl.write(unicode_to_bytes(
                             self.variable_names + defaults.outputfile_eol))
                         self._variable_names_changed = False
                         line = ''  # Skip old varnames
-                fl.write(unicode2byte(line))
+                fl.write(unicode_to_bytes(line))
             tmpfl.close()
             fl.close()
 
@@ -627,26 +634,26 @@ class EventFile(OutputFile):
         if clock is not None:
             self._clock = clock
         else:
-            if not _internals.active_exp.is_initialized:
+            if not _internals.active_exp.is_initialised:
                 raise RuntimeError(
-                    "Cannot find a clock. Initialize Expyriment!")
+                    "Cannot find a clock. initialise Expyriment!")
             self._clock = _internals.active_exp.clock
 
         try:
             display = repr(_internals.active_exp.screen.window_size)
             window_mode = repr(_internals.active_exp.screen.window_mode)
-            open_gl = repr(_internals.active_exp.screen.open_gl)
-        except:
+            opengl = repr(_internals.active_exp.screen.opengl)
+        except Exception:
             display = "unknown"
             window_mode = "unknown"
-            open_gl = "unknown"
+            opengl = "unknown"
 
         self.write_comment("sha1: {0}".format(
                                     get_experiment_secure_hash()))
         self.write_comment("modules: {0}".format(
                             module_hashes_as_string()))
-        self.write_comment("display: size={0}, window_mode={1}, open_gl={2}".format(
-            display, window_mode, open_gl))
+        self.write_comment("display: size={0}, window_mode={1}, opengl={2}".format(
+            display, window_mode, opengl))
         self.write_comment("os: {0}".format(uname()))
 
         self.write_line("Time,Type,Event,Value,Detail,Detail2")
@@ -674,7 +681,7 @@ class EventFile(OutputFile):
             the event to be logged (anything, will be casted to str)
         log_event_tag : numeral or string, optional
             if log_event_tag is defined, event file logs the inter-event-intervalls
-            and adds a summary of the intervalls at the end of the file
+            and adds a summary of the intervals at the end of the file
 
         Returns
         -------
@@ -684,7 +691,7 @@ class EventFile(OutputFile):
         """
 
         log_time = self._clock.time
-        if not _internals.is_base_string(event):
+        if not isinstance(event, (str, bytes)):
             event = str(event)
         self.write_line(repr(log_time) + self.delimiter + event)
         if log_event_tag is not None:
@@ -705,16 +712,16 @@ class EventFile(OutputFile):
         self.write_line(line)
 
     def _write_inter_event_intervall_summary(self):
-        """appending the inter event intervall summary to event file, if log_event_tag have been set while presentation
+        """appending the inter event interval summary to event file, if log_event_tag have been set while presentation
         this function will be called at exit"""
 
-        for l in self._inter_event_intervall_log.summary():
-            self.write_comment(l)
+        for line in self._inter_event_intervall_log.summary():
+            self.write_comment(line)
 
 
 
-class _InterEventIntervallLog():
-    """This class is used to log the intervalls of tagged events to get a
+class _InterEventIntervallLog:
+    """This class is used to log the intervals of tagged events to get a
     summary of the timing at the end of the event file
     """
 
@@ -732,18 +739,18 @@ class _InterEventIntervallLog():
 
         try:
             self.log_dict[event_tag].append(time)
-        except:
+        except Exception:
             self.log_dict[event_tag] = [time]
 
 
     def _get_iei_intervalls(self, from_tag, to_tag):
-        """helper function: get the intervalls between two events"""
+        """helper function: get the intervals between two events"""
 
         rtn = []
         try:
             time_from = sorted(self.log_dict[from_tag])
             time_to = sorted(self.log_dict[to_tag])
-        except:
+        except Exception:
             return rtn
 
         for f in time_from:
@@ -775,9 +782,9 @@ class _InterEventIntervallLog():
                 txt = "{0} --> {1}: n={2}".format(a,b, len(iei))
                 if len(iei)>0:
                     txt += ", mean={0}, median={1}, std={2}".format(
-                                misc.round(statistics.mean(iei),2),
-                                misc.round(statistics.median(iei),2),
-                                misc.round(statistics.std(iei),2),
+                                misc.py2_round(statistics.mean(iei),2),
+                                misc.py2_round(statistics.median(iei),2),
+                                misc.py2_round(statistics.std(iei),2),
                                 )
                 rtn.append(txt)
         return rtn
