@@ -4,31 +4,25 @@ Mouse input.
 This module contains a class implementing pygame mouse input.
 
 """
-from __future__ import absolute_import, print_function, division
-from builtins import *
-
 
 __author__ = 'Florian Krause <florian@expyriment.org>, \
 Oliver Lindemann <oliver@expyriment.org>'
-__version__ = ''
-__revision__ = ''
-__date__ = ''
-
 
 from types import FunctionType
 
 import pygame
 
-from . import defaults
-from ..misc._timer import get_time
-from ..misc import is_android_running
-from ._input_output  import Input
 from .. import _internals, misc
+from ..misc import is_android_running
+from ..misc._timer import get_time
+from . import defaults
+from ._input_output import Input
+
 
 class Mouse(Input):
     """A class implementing a mouse input.
 
-    Calling ``expyriment.control.initialize(exp)`` will automatically create a
+    Calling ``expyriment.control.initialise(exp)`` will automatically create a
     mouse instance and will reference it in exp.mouse for easy access.
 
     """
@@ -41,7 +35,7 @@ class Mouse(Input):
 
     def __init__(self, show_cursor=True, track_button_events=None,
                  track_motion_events=None):
-        """Initialize a mouse input.
+        """initialise a mouse input.
 
         Parameters
         ----------
@@ -132,18 +126,20 @@ class Mouse(Input):
         if corner is not None:
             if not isinstance(corner, int) or corner<0 or corner >3:
                 corner = None
-                print("Warning: {} is an unkown corner location. Mouse quit "
+                print("Warning: {} is an unknown corner location. Mouse quit "
                                      "event is deactivated.".format(corner))
         Mouse._quit_corner_location = corner
 
         try:
             Mouse._corner_rect_size = (int(corner_rect_size[0]),
                                        int(corner_rect_size[1]))
-        except:
+        except Exception:
             pass
 
     @staticmethod
-    def process_quit_event(click_position=None):
+    def process_quit_event(click_position=None, event_detected_function=None,
+                           quit_confirmed_function=None,
+                           quit_denied_function=None):
         """Check if mouse exit action has been performed.
 
         If quit corner location is has been defined via
@@ -162,6 +158,13 @@ class Mouse(Input):
             clicked location to be processed. If not defined, the Pygame event
             queue will be checked for mouse down events and the current
             position is taken
+        event_detected_function : function, optional
+            function to be called when key event is detected.
+        quit_confirmed_function : function, optional
+            function to be called if quitting has been confirmed.
+        quit_denied_function : function, optional
+            function to be called if quitting has been denied.
+
 
         Returns
         -------
@@ -179,7 +182,7 @@ class Mouse(Input):
             return False
 
         if click_position is None:
-            # check Pygame queu
+            # check Pygame queue
             pos = None
             # pygame.event.pump() # not sure if it is required!
             for event in pygame.event.get(pygame.MOUSEBUTTONDOWN):
@@ -230,7 +233,10 @@ class Mouse(Input):
                         pygame.KEYDOWN,
                         {'key': _internals.active_exp.keyboard.get_quit_key()})
                     return _internals.active_exp.keyboard.process_control_keys(
-                        key_event=simulated_key)
+                        key_event=simulated_key,
+                        event_detected_function=event_detected_function,
+                        quit_confirmed_function=quit_confirmed_function,
+                        quit_detected_function=quit_denied_function)
         return False
 
 
@@ -442,7 +448,8 @@ class Mouse(Input):
 
     def wait_event(self, wait_button=True, wait_motion=True, buttons=None,
                    duration=None, wait_for_buttonup=False,
-                   callback_function=None, process_control_events=True):
+                   callback_function=None, process_control_events=True,
+                   low_performance=False):
         """Wait for a mouse event (i.e., motion, button press or wheel event).
 
         Button id coding:
@@ -469,11 +476,14 @@ class Mouse(Input):
         process_control_events : bool, optional
             process ``io.keyboard.process_control_keys()`` and
             ``io.mouse.process_quit_event()`` (default = True)
+        low_performance : bool, optional
+            reduce CPU performance (and allow potential threads to run) while
+            waiting at the cost of less timing accuracy (default = False)
 
         Returns
         -------
         event_id : int
-            id of the event that quited waiting
+            id of the event that quit waiting
         move : bool
             True if a motion occurred
         pos : (int, int)
@@ -500,13 +510,13 @@ class Mouse(Input):
         old_pos = pygame.mouse.get_pos()
         btn_id = None
         rt = None
-        motion_occured = False
+        motion_occurred = False
         if buttons is None:
             buttons = [0, 1, 2, 3, 4]
         else:
             try:
                 buttons = list(buttons)
-            except:
+            except Exception:
                 buttons = [buttons]
         while True:
             if isinstance(callback_function, FunctionType):
@@ -515,7 +525,7 @@ class Mouse(Input):
                     btn_id = rtn_callback
                     rt = int((get_time() - start) * 1000)
                     break
-            if _internals.active_exp.is_initialized:
+            if _internals.active_exp.is_initialised:
                 rtn_callback = _internals.active_exp._execute_wait_callback()
                 if isinstance(rtn_callback, _internals.CallbackQuitEvent):
                     btn_id = rtn_callback
@@ -525,7 +535,7 @@ class Mouse(Input):
                     if _internals.active_exp.keyboard.process_control_keys():
                         break
             if wait_motion:
-                motion_occured = old_pos != pygame.mouse.get_pos()
+                motion_occurred = old_pos != pygame.mouse.get_pos()
             if wait_button:
                 if wait_for_buttonup:
                     btn_id = self.get_last_button_up_event()
@@ -535,23 +545,26 @@ class Mouse(Input):
             if btn_id ==-1:
                 btn_id = None
                 break
-            elif btn_id in buttons or motion_occured:
+            elif btn_id in buttons or motion_occurred:
                 rt = int((get_time() - start) * 1000)
                 break
             elif (duration is not None and \
                     int((get_time() - start) * 1000) >= duration):
                 break
+            if low_performance:
+                _internals.low_performance_sleep()
 
         position_in_expy_coordinates = self.position
 
         if self._logging:
             _internals.active_exp._event_file_log(
-            "Mouse,received,{0}-{1},wait_event".format(btn_id, motion_occured))
-        return btn_id, motion_occured, position_in_expy_coordinates, rt
+            "Mouse,received,{0}-{1},wait_event".format(btn_id, motion_occurred))
+        return btn_id, motion_occurred, position_in_expy_coordinates, rt
 
 
     def wait_press(self, buttons=None, duration=None, wait_for_buttonup=False,
-                   callback_function=None, process_control_events=True):
+                   callback_function=None, process_control_events=True,
+                   low_performance=False):
         """Wait for a mouse button press or mouse wheel event.
 
         Parameters
@@ -567,11 +580,14 @@ class Mouse(Input):
         process_control_events : bool, optional
             process ``io.keyboard.process_control_keys()`` and
             ``io.mouse.process_quit_event()`` (default = false)
+        low_performance : bool, optional
+            reduce CPU performance (and allow potential threads to run) while
+            waiting at the cost of less timing accuracy (default = False)
 
         Returns
         -------
         event_id : int
-            id of the event that quited waiting
+            id of the event that quit waiting
         pos : (int, int)
             mouse position (tuple)
         rt : int
@@ -592,11 +608,12 @@ class Mouse(Input):
                               buttons=buttons, duration=duration,
                               wait_for_buttonup=wait_for_buttonup,
                               callback_function=callback_function,
-                              process_control_events=process_control_events)
+                              process_control_events=process_control_events,
+                              low_performance=low_performance)
         return rtn[0], rtn[2], rtn[3]
 
     def wait_motion(self, duration=None, callback_function=None,
-                    process_control_events=True):
+                    process_control_events=True, low_performance=False):
         """Wait for a mouse motion.
 
         Parameters
@@ -608,6 +625,9 @@ class Mouse(Input):
         process_control_events : bool, optional
             process ``io.keyboard.process_control_keys()`` and
             ``io.mouse.process_quit_event()`` (default = false)
+        low_performance : bool, optional
+            reduce CPU performance (and allow potential threads to run) while
+            waiting at the cost of less timing accuracy (default = False)
 
         Returns
         -------
@@ -621,7 +641,8 @@ class Mouse(Input):
         rtn = self.wait_event(wait_button=False, wait_motion=True, buttons=[],
                               duration=duration, wait_for_buttonup=False,
                               callback_function=callback_function,
-                              process_control_events=process_control_events)
+                              process_control_events=process_control_events,
+                              low_performance=low_performance)
 
         if isinstance(rtn[0], _internals.CallbackQuitEvent):
             return rtn[0], rtn[3]
@@ -677,6 +698,7 @@ class Mouse(Input):
         """
 
         from .. import stimuli
+
         # measure mouse polling time
         info = """This will test how timing accurate your mouse is.
 
