@@ -19,18 +19,12 @@ Each instance of the class can provide access to a different parallel
 port.
 
 """
-from __future__ import absolute_import, print_function, division
-from builtins import *
 
 __author__ = 'Florian Krause <florian@expyriment.org> \
 Oliver Lindemann <oliver@expyriment.org>'
-__version__ = ''
-__revision__ = ''
-__date__ = ''
 
-
-import sys
 import os
+import sys
 
 from ... import _internals, misc
 from ...io._input_output import Input, Output
@@ -40,7 +34,7 @@ if sys.platform.startswith('linux'):
         from ._linux import PParallelLinux
         _ParallelPort = PParallelLinux
         _ParallelPort._driver = "pyparallel"
-    except:
+    except Exception:
         _ParallelPort = None
 elif sys.platform == 'win32':
     try:
@@ -50,17 +44,17 @@ elif sys.platform == 'win32':
         try:
             windll.inpout32
             _ParallelPort._driver = "inpout32"
-        except:
+        except Exception:
             windll.inpoutx64
             _ParallelPort._driver = "inpoutx64"
-    except:
+    except Exception:
         try:
             from ._dlportio import PParallelDLPortIO
             _ParallelPort = PParallelDLPortIO
             from ctypes import windll
             windll.dlportio
             _ParallelPort._driver = "dlportio"
-        except:
+        except Exception:
             _ParallelPort = None
 else: # MAC
     _ParallelPort = None
@@ -94,10 +88,10 @@ class ParallelPort(Input, Output):
     Notes
     -----
     CAUTION: On Windows, one of the following parallel port drivers needs to
-    be installed: 'inpout32' (http://www.highrez.co.uk/Downloads/InpOut32/) or
-    'dlportio' (http://real.kiev.ua/2010/11/29/dlportio-and-32-bit-windows/).
+    be installed: 'inpout32' (https://www.highrez.co.uk/Downloads/InpOut32/) or
+    'dlportio' (https://real.kyiv.ua/2010/11/29/dlportio-and-32-bit-windows/).
     On Linux, the Python package 'PyParallel'
-    (http://pyserial.sourceforge.net/pyparallel.html) has to be installed.
+    (https://pypi.org/project/pyparallel/) has to be installed.
 
     """
 
@@ -129,13 +123,13 @@ class ParallelPort(Input, Output):
         if _ParallelPort is None:
             if sys.platform == "win32":
                 _message = "Please install one of the following parallel port " + \
-"drivers: 'inpout32' (http://www.highrez.co.uk/Downloads/InpOut32/) or " + \
-"'dlportio' (http://real.kiev.ua/2010/11/29/dlportio-and-32-bit-windows/)."
+"drivers: 'inpout32' (https://www.highrez.co.uk/Downloads/InpOut32/) or " + \
+"'dlportio' (https://real.kyiv.ua/2010/11/29/dlportio-and-32-bit-windows/)."
             elif sys.platform.startswith("linux"):
                 _message = "Please install the Python package 'PyParallel'."
             else:
                 _message = "Not available on your computer."
-            message = "ParallelPort cannot be initialized! {0}".format(
+            message = "ParallelPort cannot be initialised! {0}".format(
                 _message)
             raise ImportError(message)
 
@@ -146,7 +140,7 @@ class ParallelPort(Input, Output):
         self._address = address
         try:
             self._parallel = _ParallelPort(address=address)
-        except:
+        except Exception:
             raise RuntimeError(
                 "Could not initiate parallel port at {0}".format(address))
         self.input_history = False  # dummy
@@ -285,11 +279,11 @@ class ParallelPort(Input, Output):
             Pin:  17    16    14     1     9     8     7     6     5
                    |     |     |     |     |     |     |     |     |
             Bit: 65536 32768 16384 8192  4096  2048  1024   512   256
-             
+
             Pin:  4    3    2   11   10   12   13   15
                   |    |    |    |    |    |    |    |
             Bit: 128  64   32   16    8    4    2    1
-             
+
         Returns
         -------
         data : int (0-65535)
@@ -426,18 +420,18 @@ class ParallelPort(Input, Output):
     @staticmethod
     def get_driver():
         """Return the driver used for interacting with parallel ports.
-        
+
         On Windows, one of the following parallel port drivers needs to
-        be installed: 'input32' (http://www.highrez.co.uk/Downloads/InpOut32/) or
-        'dlportio' (http://real.kiev.ua/2010/11/29/dlportio-and-32-bit-windows/).
+        be installed: 'input32' (https://www.highrez.co.uk/Downloads/InpOut32/) or
+        'dlportio' (https://real.kyiv.ua/2010/11/29/dlportio-and-32-bit-windows/).
         On Linux, the Python package 'PyParallel'
-        (http://pyserial.sourceforge.net/pyparallel.html) has to be installed.
-    
+        (https://pypi.org/project/pyparallel/) has to be installed.
+
         """
-        
+
         if _ParallelPort is not None:
             return _ParallelPort._driver
-        
+
     @staticmethod
     def get_available_ports():
         """Return an array of strings representing available parallel ports.
@@ -453,15 +447,11 @@ class ParallelPort(Input, Output):
 
         """
 
-        ports = []
         if sys.platform.startswith("linux"):
             dev = os.listdir('/dev')
-            for p in dev:
-                if p.startswith("parport"):
-                    ports.append(p)
+            ports = sorted(p for p in dev if p.startswith("parport"))
         else:
-            pass
-        ports.sort()
+            ports = []
 
         return ports
 
@@ -469,7 +459,7 @@ class ParallelPort(Input, Output):
     def _self_test(exp):
         """Test the parallel port."""
 
-        from ... import stimuli, io
+        from ... import io, stimuli
         result = {}
         result["testsuite_parallel_port"] = ""
         result["testsuite_parallel_success"] = "No"
@@ -477,15 +467,15 @@ class ParallelPort(Input, Output):
         if _ParallelPort is None:
             if sys.platform == "win32":
                 _message = "Please install one of the following parallel port " + \
-"drivers: 'inpout32' (http://www.highrez.co.uk/Downloads/InpOut32/) or " + \
-"'dlportio' (http://real.kiev.ua/2010/11/29/dlportio-and-32-bit-windows/)."
+"drivers: 'inpout32' (https://www.highrez.co.uk/Downloads/InpOut32/) or " + \
+"'dlportio' (https://real.kyiv.ua/2010/11/29/dlportio-and-32-bit-windows/)."
             elif sys.platform.startswith("linux"):
                 _message = "Please install the Python package 'PyParallel'."
             else:
                 _message = "Not available on your computer."
             while True:
                 stimuli.TextScreen(
-                    "ParallelPort cannot be initialized!",
+                    "ParallelPort cannot be initialised!",
                     _message + "\n\n[Press RETURN to continue]").present()
                 key, rt_ = exp.keyboard.wait(misc.constants.K_RETURN)
                 if key is not None:
@@ -497,7 +487,7 @@ class ParallelPort(Input, Output):
                    "communication works correctly.\n\n" \
                    "You will be asked to enter a port address. " \
                    "On Windows this will be a hexadecimal number (e.g. " \
-                   "'0x0378', which is ofter the address of LPT1). " \
+                   "'0x0378', which is often the address of LPT1). " \
                    "On Linux it will be of the form '/dev/parportX', "\
                    "where X denotes the number of the port.\n\n" \
                    "Once the port is opened, a list of control, status and " \
@@ -518,7 +508,7 @@ class ParallelPort(Input, Output):
             address = io.TextInput("Port address:").get()
             try:
                 pp = io.ParallelPort(address=address)
-            except:
+            except Exception:
                 while True:
                     stimuli.TextScreen(
                         "Could not open port address {0}!".format(address),
@@ -530,13 +520,13 @@ class ParallelPort(Input, Output):
 
             pin_canvas = stimuli.Canvas((800, 600))
             poll = stimuli.TextLine("Poll", text_colour=[255, 255, 255],
-                                    position=[0, 165])
+                                    text_size=20, position=[0, 165])
             poll.plot(pin_canvas)
             poll_line = stimuli.Rectangle((660, 2), colour=[255, 255, 255],
                                           position=[0, 120])
             poll_line.plot(pin_canvas)
             control = stimuli.TextLine("Control", text_colour=[100, 100, 100],
-                                       position=[-260, 95])
+                                       text_size=20, position=[-260, 95])
             control.plot(pin_canvas)
             control_line = stimuli.Rectangle((140, 2), colour=[100, 100, 100],
                                              position=[-260, 50])
@@ -544,6 +534,7 @@ class ParallelPort(Input, Output):
             data = stimuli.TextLine(
                 "Data",
                 text_colour=misc.constants.C_EXPYRIMENT_ORANGE,
+                text_size=20,
                 position=[-20, 95])
             data.plot(pin_canvas)
             data_line = stimuli.Rectangle(
@@ -554,6 +545,7 @@ class ParallelPort(Input, Output):
             status = stimuli.TextLine(
                 "Status",
                 text_colour=misc.constants.C_EXPYRIMENT_PURPLE,
+                text_size=20,
                 position=[240, 95])
             status.plot(pin_canvas)
             status_line = stimuli.Rectangle(
@@ -581,7 +573,7 @@ class ParallelPort(Input, Output):
                     bg.plot(pin_canvas)
                     colour = _internals.active_exp._background_colour
                 stim = stimuli.TextLine(repr(pin), text_font="freemono",
-                                        text_colour=colour,
+                                        text_colour=colour, text_size=20,
                                         position=[x_pos, 0])
                 stim.plot(pin_canvas)
                 pins[pin] = stim
@@ -591,7 +583,7 @@ class ParallelPort(Input, Output):
                 x_pos += 40
 
             control2 = stimuli.TextLine("Control", text_colour=[100, 100, 100],
-                                        position=[-260, -95])
+                                        text_size=20, position=[-260, -95])
             control2.plot(pin_canvas)
             control2_line = stimuli.Rectangle((140, 2), colour=[100, 100, 100],
                                               position=[-260, -50])
@@ -600,6 +592,7 @@ class ParallelPort(Input, Output):
             data2 = stimuli.TextLine(
                 "Data",
                 text_colour=misc.constants.C_EXPYRIMENT_ORANGE,
+                text_size=20,
                 position=[-20, -95])
             data2.plot(pin_canvas)
             data2_line = stimuli.Rectangle(
@@ -609,7 +602,7 @@ class ParallelPort(Input, Output):
             data2_line.plot(pin_canvas)
 
             send = stimuli.TextLine("Send", text_colour=[255, 255, 255],
-                                    position=[-100, -165])
+                                    text_size=20, position=[-100, -165])
             send.plot(pin_canvas)
             send_line = stimuli.Rectangle((460, 2), colour=[255, 255, 255],
                                           position=[-100, -120])
@@ -628,7 +621,7 @@ class ParallelPort(Input, Output):
             def _update(inputs_states, outputs_states, read_control,
                         read_data, read_status, read_poll):
                 for pin in list(inputs.keys()):
-                    if inputs_states[pin] == True:
+                    if inputs_states[pin] is True:
                         inputs[pin].colour = [0, 255, 0]
                     else:
                         inputs[pin].colour = [0, 30, 0]
@@ -638,7 +631,7 @@ class ParallelPort(Input, Output):
 
                 send = 0
                 for pin in list(outputs.keys()):
-                    if outputs_states[pin][0] == True:
+                    if outputs_states[pin][0] is True:
                         send += outputs_states[pin][1]
                         outputs[pin].colour = [255, 0, 0]
                     else:
@@ -655,18 +648,21 @@ class ParallelPort(Input, Output):
                 rc = stimuli.TextLine(
                     "{0}".format(read_control),
                     text_colour=[0, 255, 0],
+                    text_size=20,
                     position=[-260, 0])
                 rc.plot(results_canvas)
 
                 rd = stimuli.TextLine(
                     "{0}".format(read_data),
                     text_colour=[0, 255, 0],
+                    text_size=20,
                     position=[-20, 0])
                 rd.plot(results_canvas)
 
                 rs = stimuli.TextLine(
                     "{0}".format(read_status),
                     text_colour=[0, 255, 0],
+                    text_size=20,
                     position=[240, 0])
                 rs.plot(results_canvas)
 
@@ -677,7 +673,7 @@ class ParallelPort(Input, Output):
 
                 pd = stimuli.TextLine(
                     "{0}".format(read_poll),
-                    text_colour=[0, 255, 0])
+                    text_colour=[0, 255, 0], text_size=20)
                 pd.plot(poll_canvas)
 
                 poll_canvas.present(clear=False, update=False)
@@ -700,12 +696,14 @@ class ParallelPort(Input, Output):
                 sc = stimuli.TextLine(
                     "{0}".format(c),
                     text_colour=[255, 0, 0],
+                    text_size=20,
                     position=[-260, 0])
                 sc.plot(results_canvas2)
 
                 sd = stimuli.TextLine(
                     "{0}".format(send & 255),
                     text_colour=[255, 0, 0],
+                    text_size=20,
                     position=[-20, 0])
                 sd.plot(results_canvas2)
 
@@ -716,7 +714,7 @@ class ParallelPort(Input, Output):
 
                 ss = stimuli.TextLine(
                     "{0}".format(send),
-                    text_colour=[255, 0, 0])
+                    text_colour=[255, 0, 0], text_size=20)
                 ss.plot(send_canvas)
 
                 if pp.reverse is True:
@@ -727,6 +725,7 @@ class ParallelPort(Input, Output):
                         position=[240, -95])
                     reverse = stimuli.TextLine(
                         "Reverse Mode",
+                        text_size=20,
                         text_colour=misc.constants.C_EXPYRIMENT_ORANGE,
                         background_colour=_internals.active_exp.background_colour)
                     reverse.plot(r)
@@ -738,6 +737,7 @@ class ParallelPort(Input, Output):
                         position=[240, -95])
                     reverse = stimuli.TextLine(
                         "Reverse Mode",
+                        text_size=20,
                         text_colour=_internals.active_exp.background_colour,
                         background_colour=_internals.active_exp.background_colour)
                     reverse.plot(r)
