@@ -5,30 +5,25 @@ This module contains a class implementing serial port input/output.
 
 """
 
-from __future__ import absolute_import, print_function, division
-from builtins import *
-
 __author__ = 'Florian Krause <florian@expyriment.org> \
 Oliver Lindemann <oliver@expyriment.org>'
-__version__ = ''
-__revision__ = ''
-__date__ = ''
-
 
 import atexit
-from types import ModuleType, FunctionType
+from types import FunctionType, ModuleType
 
 try:
     import serial
     from serial.tools.list_ports import comports as list_com_ports
-except:
+except Exception:
     serial = None
     list_com_ports = None
 
+import pygame
+
+from .. import __version__, _internals, misc
+from .._internals import CallbackQuitEvent
 from . import defaults
 from ._input_output import Input, Output
-from .. import _internals, misc
-from .._internals import CallbackQuitEvent, PYTHON3
 
 
 class SerialPort(Input, Output):
@@ -84,7 +79,7 @@ class SerialPort(Input, Output):
         """
 
         if not isinstance(serial, ModuleType):
-            message = """SerialPort can not be initialized.
+            message = """SerialPort can not be initialised.
 The Python package 'pySerial' is not installed."""
             raise ImportError(message)
 
@@ -118,7 +113,7 @@ The Python package 'pySerial' is not installed."""
         if clock is not None:
             self._clock = clock
         else:
-            if _internals.active_exp.is_initialized:
+            if _internals.active_exp.is_initialised:
                 self._clock = _internals.active_exp.clock
             else:
                 self._clock = misc.Clock()
@@ -138,7 +133,7 @@ The Python package 'pySerial' is not installed."""
                                     stopbits, timeout, xonxoff, rtscts,
                                     dsrdtr)
         if not self._serial.isOpen():
-            raise IOError("Could not open serial port")
+            raise OSError("Could not open serial port")
 
         atexit.register(self.close)
 
@@ -265,12 +260,12 @@ The Python package 'pySerial' is not installed."""
 
         try:
             self._serial.close()
-        except:
+        except Exception:
             pass
 
     @property
     def has_input_history(self):
-        """Returns if a input_history exists or not (True / False)."""
+        """Returns if an input_history exists or not (True / False)."""
 
         return isinstance(self._input_history, misc.ByteBuffer)
 
@@ -305,7 +300,7 @@ The Python package 'pySerial' is not installed."""
     def read_input(self):
         """Read all input from serial port.
 
-        If a input_history is used, all received data will be added.
+        If an input_history is used, all received data will be added.
 
         Returns
         -------
@@ -340,7 +335,7 @@ The Python package 'pySerial' is not installed."""
     def poll(self):
         """Poll the serial port.
 
-        If a input_history is used, it will be added.
+        If an input_history is used, it will be added.
 
         Returns
         -------
@@ -402,7 +397,7 @@ The Python package 'pySerial' is not installed."""
         """
 
         if _internals.skip_wait_methods:
-            return
+            return None
 
         rtn_string = b""
         if duration is not None:
@@ -419,7 +414,7 @@ The Python package 'pySerial' is not installed."""
                 if isinstance(rtn_callback, CallbackQuitEvent):
                     rtn_string = rtn_callback
                     break
-            if _internals.active_exp.is_initialized:
+            if _internals.active_exp.is_initialised:
                 rtn_callback = _internals.active_exp._execute_wait_callback()
                 if isinstance(rtn_callback, CallbackQuitEvent):
                     rtn_string = rtn_callback
@@ -429,7 +424,7 @@ The Python package 'pySerial' is not installed."""
                        _internals.active_exp.keyboard.process_control_keys():
                         break
                 else:
-                    _internals.pump_pygame_events()
+                    pygame.event.pump()
 
             byte = self.poll()
             if byte:
@@ -465,8 +460,7 @@ The Python package 'pySerial' is not installed."""
 
         if not isinstance(serial, ModuleType):
             return None
-        ports = sorted([x[0] for x in list_com_ports()])
-        return ports
+        return sorted([x[0] for x in list_com_ports()])
 
     def send(self, data):
         """Send data via the serial port.
@@ -498,10 +492,7 @@ The Python package 'pySerial' is not installed."""
 
         """
 
-        if PYTHON3:
-            data = list(data)
-        else:
-            data = [ord(x) for x in data]
+        data = list(data)
 
         if carriage_return:
             data.append(13)
@@ -559,8 +550,17 @@ The Python package 'pySerial' is not installed."""
         else:
             import serial
 
-            idx = io.TextMenu("Select Serial Port", ports, width=200,
-                                         justification=1, scroll_menu=5).get()
+            # Scale fonts/logo according to default font size
+            scaling = exp.text_size / 20
+            if scaling < 1:
+                scaling = 1
+            if scaling > 2:
+                scaling = 2
+
+            idx = io.TextMenu("Select Serial Port", ports,
+                              width=int(200 * scaling),
+                              text_size=int(20 * scaling), justification=1,
+                              scroll_menu=5).get()
             comport = ports[idx]
 
             rates = [0, 50, 75, 110, 134, 150, 200, 300, 600, 1200, 1800, 2400,
@@ -568,19 +568,22 @@ The Python package 'pySerial' is not installed."""
                      500000, 576000, 921600, 1000000, 1152000, 1500000, 2000000,
                      2500000, 3000000, 3500000, 4000000]
             idx = io.TextMenu("Select Baudrate", [repr(x) for x in rates],
-                                         width=200, justification=1,
-                                         scroll_menu=2).get(preselected_item=14)
+                              width=int(200 * scaling),
+                              text_size=int(20 * scaling), justification=1,
+                              scroll_menu=2).get(preselected_item=14)
             baudrate = rates[idx]
 
             parities = ["N", "E", "O"]
-            idx = io.TextMenu("Select Parity", parities, width=200,
-                                         justification=1, scroll_menu=2).get(
-                                             preselected_item=0)
+            idx = io.TextMenu("Select Parity", parities,
+                              width=int(200 * scaling),
+                              text_size=int(20 * scaling), justification=1,
+                              scroll_menu=2).get(preselected_item=0)
             parity = parities[idx]
 
             stopbits = [0, 1, 1.5, 2]
             idx = io.TextMenu("Select Stopbits", [repr(x) for x in stopbits],
-                              width=200, justification=1,
+                              width=int(200 * scaling), justification=1,
+                              text_size=int(20 * scaling),
                               scroll_menu=2).get(preselected_item=1)
             stopbit = stopbits[idx]
 
@@ -620,6 +623,8 @@ The Python package 'pySerial' is not installed."""
                 else:
                     s.present()
 
+            ser.close()
+
             result["testsuite_serial_port"] = comport
             result["testsuite_serial_baudrate"] = baudrate
             result["testsuite_serial_parity"] = parity
