@@ -4,37 +4,23 @@ A text input box.
 This module contains a class implementing a text input box for user input.
 
 """
-from __future__ import absolute_import, print_function, division
-from builtins import (ascii, bytes, chr, dict, filter, hex, input,
-                      int, map, next, oct, pow, range, round,
-                      str, super, zip)  # without open, because
-                       # pygame.font.Font needs old file object under PY2
-
 
 __author__ = 'Florian Krause <florian@expyriment.org>, \
 Oliver Lindemann <oliver@expyriment.org>'
-__version__ = ''
-__revision__ = ''
-__date__ = ''
-
 
 import pygame
-try:
-    import android
-except ImportError:
-    android = None
 
 try:
-    import android.show_keyboard as android_show_keyboard
+    import android
     import android.hide_keyboard as android_hide_keyboard
+    import android.show_keyboard as android_show_keyboard
 except ImportError:
     android_show_keyboard = android_hide_keyboard = None
 
-from . import defaults
 from .. import _internals, stimuli
-from ..misc import find_font, unicode2byte, constants, \
-                 numpad_digit_code2ascii
 from .._internals import CallbackQuitEvent
+from ..misc import constants, find_font, is_android_running, numpad_digit_code_to_ascii
+from . import defaults
 from ._input_output import Input
 
 
@@ -102,9 +88,9 @@ class TextInput(Input):
 
         """
 
-        if not _internals.active_exp.is_initialized:
+        if not _internals.active_exp.is_initialised:
             raise RuntimeError(
-                "Cannot create TextInput before expyriment.initialize()!")
+                "Cannot create TextInput before expyriment.initialise()!")
         Input.__init__(self)
         self._message = message
         if position is not None:
@@ -143,8 +129,8 @@ class TextInput(Input):
         try:
             with open(self._message_font, 'rb') as f:
                 pygame.font.Font(f, 10)
-        except:
-            raise IOError("Font '{0}' not found!".format(message_font))
+        except Exception:
+            raise OSError("Font '{0}' not found!".format(message_font))
         if message_bold is not None:
             self._message_bold = message_bold
         else:
@@ -178,8 +164,8 @@ class TextInput(Input):
         try:
             with open(self._user_text_font, 'rb') as f:
                 pygame.font.Font(f, 10)
-        except:
-            raise IOError("Font '{0}' not found!".format(user_text_font))
+        except Exception:
+            raise OSError("Font '{0}' not found!".format(user_text_font))
         if user_text_colour is None:
             user_text_colour = defaults.textinput_user_text_colour
         if user_text_colour is not None:
@@ -345,6 +331,11 @@ class TextInput(Input):
                 return rtn_callback, None
 
             events = pygame.event.get(pygame.KEYDOWN)
+            # Clear keyup events to prevent limit on unicode field storage:
+            # https://github.com/pygame/pygame/issues/3229
+            pygame.event.get(pygame.KEYUP)
+
+            keys = []
             for event in events:
                 if process_control_events:
                     if _internals.active_exp.keyboard.process_control_keys(event) or \
@@ -352,7 +343,8 @@ class TextInput(Input):
                         self._create()
                         self._update()
                         return None, None
-                return event.key, event.unicode
+                keys.append((event.key, event.unicode))
+            return keys
 
     def _create(self):
         """Create the input box."""
@@ -410,7 +402,7 @@ class TextInput(Input):
         self._canvas.plot(background)
         background.present()
         background.present()  # for flipping with double buffer
-        background.present()  # for flipping with tripple buffer
+        background.present()  # for flipping with triple buffer
 
     def _update(self):
         """Update the input box."""
@@ -446,7 +438,8 @@ class TextInput(Input):
             user_canvas._get_surface().blit(user_text._get_surface(), (0, 2))
         user_canvas.present(clear=False)
 
-    def get(self, default_input="", process_control_events=True):
+    def get(self, default_input="", clear_event_cue=True,
+            process_control_events=True):
         """Get input from user.
 
         Notes
@@ -458,6 +451,8 @@ class TextInput(Input):
         ----------
         default_input : str, optional
             default input in the textbox
+        clear_event_cue : bool, optional
+            clear keyboard events from the event cue (default = True)
         process_control_events : bool, optional
             process ``io.Keyboard.process_control_keys()`` and
             ``io.Mouse.process_quit_event()`` (default = True)
@@ -475,6 +470,9 @@ class TextInput(Input):
 
         """
 
+        if clear_event_cue:
+            _internals.active_exp.keyboard.clear()
+
         if android_show_keyboard is not None:
             android_show_keyboard()
         self._user = []
@@ -487,27 +485,31 @@ class TextInput(Input):
         else:
             filter = self._character_filter
 
-        while True:
-            inkey, string = self._get_key(process_control_events)
-            if isinstance(inkey, CallbackQuitEvent):
-                return None
-            elif inkey == pygame.K_BACKSPACE:
-                self._user = self._user[0:-1]
-            elif inkey == pygame.K_RETURN or inkey == pygame.K_KP_ENTER:
-                break
-            elif inkey not in (pygame.K_LCTRL, pygame.K_RCTRL, pygame.K_TAB):
-                if not self._user_text_surface_size[0] >= self._max_size[0]:
-                    if android is not None:
-                        if inkey in filter:
+        done = False
+        while not done:
+            keys = self._get_key(process_control_events)
+            for inkey, string in keys:
+                if isinstance(inkey, CallbackQuitEvent):
+                    return None
+                elif inkey == pygame.K_BACKSPACE:
+                    self._user = self._user[0:-1]
+                elif inkey == pygame.K_RETURN or inkey == pygame.K_KP_ENTER:
+                    done = True
+                    break
+                elif inkey not in (pygame.K_LCTRL, pygame.K_RCTRL,
+                                   pygame.K_TAB):
+                    if not self._user_text_surface_size[0] >= self._max_size[0]:
+                        if is_android_running():
+                            if inkey in filter:
+                                if inkey in constants.K_ALL_KEYPAD_DIGITS:
+                                    inkey = numpad_digit_code_to_ascii(inkey)
+                                self._user.append(chr(inkey))
+                        else:
                             if inkey in constants.K_ALL_KEYPAD_DIGITS:
-                                inkey = numpad_digit_code2ascii(inkey)
-                            self._user.append(chr(inkey))
-                    else:
-                        if inkey in constants.K_ALL_KEYPAD_DIGITS:
-                            self._user.append(chr(numpad_digit_code2ascii(
-                                inkey)))
-                        elif string and ord(string) in filter:
-                            self._user.append(string)
+                                self._user.append(chr(numpad_digit_code_to_ascii(
+                                    inkey)))
+                            elif string and ord(string) in filter:
+                                self._user.append(string)
             self._update()
         got = "".join(self._user)
         if self._logging:
@@ -524,7 +526,7 @@ class TextInput(Input):
             from .. import control
             control.set_develop_mode(True)
             control.defaults.event_logging = 0
-            exp_ = control.initialize()
+            _exp = control.initialise()
         textinput = TextInput(message="Subject Number:",
                               message_colour=(160, 70, 250),
                               user_text_size=30,
