@@ -1,22 +1,17 @@
 """
+info["hardware_displays"] =
 Get System Information.
 """
-from __future__ import absolute_import, print_function, division
-from future import standard_library
-standard_library.install_aliases()
-from builtins import *
 
 __author__ = 'Florian Krause <florian@expyriment.org>, \
 Oliver Lindemann <oliver@expyriment.org>'
-__version__ = ''
-__revision__ = ''
-__date__ = ''
 
-import sys
 import os
 import platform
-import subprocess
 import socket
+import subprocess
+import sys
+
 try:
     import OpenGL as _ogl
 except ImportError:
@@ -30,39 +25,58 @@ except ImportError:
     _com_port_list = []
 try:
     import parallel as _parallel
-except ImportError:
-    _parallel = None
-except WindowsError:
+except Exception:
     _parallel = None
 try:
     import numpy as _numpy
-except:
+except Exception:
     _numpy = None
 try:
     import mediadecoder as _mediadecoder
-except:
+except Exception:
     _mediadecoder = None
 try:
     import sounddevice as _sounddevice
-except:
+except Exception:
     _sounddevice = None
 try:
     import PIL.Image as _pil
-except:
+except Exception:
     _pil = None
 import pygame
 
-from .._internals import PYTHON3
+from .._internals import get_version
+
+
+def _get_registry_subkeys(key, subkey):
+    try:
+        import winreg as _winreg
+    except Exception:
+        return ""
+
+    key_id = getattr(_winreg, key)
+    handle = _winreg.OpenKey(key_id, subkey)
+    subkeys = []
+    i = 0
+    while True:
+        try:
+            subkeys.append(str(_winreg.EnumKey(handle, i)))
+            i += 1
+        except OSError:
+            break
+    return subkeys
+
 
 def _get_registry_value(key, subkey, value):
-    if PYTHON3:
-        import winreg as _winreg  # TODO check me on Windows
-    else:
-        import _winreg
+    try:
+        import winreg as _winreg
+    except Exception:
+        return ""
+
     key = getattr(_winreg, key)
     handle = _winreg.OpenKey(key, subkey)
     (value, type) = _winreg.QueryValueEx(handle, value)
-    return value
+    return str(value)
 
 
 def get_system_info(as_string=False):
@@ -75,25 +89,61 @@ def get_system_info(as_string=False):
 
     """
 
-    from ..io import SerialPort, ParallelPort
     from .._internals import get_settings_folder
+    from ..io import ParallelPort
+    from ._miscellaneous import get_display_info
+    try:
+        from platform import linux_distribution
+    except Exception:
+        try:
+            from distro import (
+                linux_distribution,  # TODO: only available for Linux, should it be a suggested package dependency?
+            )
+        except Exception:
+            def linux_distribution():
+                name = ""
+                version = ""
+                name_found = False
+                version_found = False
+                files = [x for x in ("/etc/os-release", "/etc/lsb-release") \
+                         if os.path.isfile(x)]
+                for release_file in files:
+                    with open(release_file) as f:
+                        for line in f:
+                            if not name_found and \
+                                    line.startswith(("NAME=", "DISTRIB_ID=")):
+                                name = line.split("=")[-1].strip().strip('"')
+                                name_found = True
+                            if not version_found and \
+                                    line.startswith(("VERSION_ID=", "DISTRIB_RELEASE=")):
+                                version = line.split("=")[-1].strip().strip('"')
+                                version_found = True
+                return (name, version)
 
     info = {}
 
     # Get platform specific info for Linux
     if sys.platform.startswith("linux"):
         os_platform = "Linux"
-        os_name = platform.linux_distribution()[0]
+        os_name = linux_distribution()[0]
+        os_version = linux_distribution()[1]
+
         details = []
         if "XDG_CURRENT_DESKTOP" in os.environ:
             details.append(os.environ["XDG_CURRENT_DESKTOP"])
-        if "DESKTOP_SESSION" in os.environ:
+        elif "DESKTOP_SESSION" in os.environ:
             details.append(os.environ["DESKTOP_SESSION"])
+        else:
+            pass
+        if "WAYLAND_DISPLAY" in os.environ:
+            details.append(os.environ["WAYLAND_DISPLAY"])
+        if "DISPLAY" in os.environ:
+            details.append(os.environ["DISPLAY"])
+
         if details != []:
             os_details = ", ".join(details)
         else:
             os_details = ""
-        os_version = platform.linux_distribution()[1]
 
         try:
             hardware_cpu_details = ""
@@ -101,7 +151,7 @@ def get_system_info(as_string=False):
                 for line in f:
                     if line.startswith("model name"):
                         hardware_cpu_details = line.split(":")[1].strip()
-        except:
+        except Exception:
             hardware_cpu_details = ""
 
         try:
@@ -123,20 +173,21 @@ def get_system_info(as_string=False):
             hardware_memory_free = str(mem_free + mem_buffers + mem_cached) + \
                                    " MB"
 
-        except:
+        except Exception:
             hardware_memory_total = ""
             hardware_memory_free = ""
 
         try:
             hardware_audio_card = ""
             cards = []
-            p = subprocess.Popen(['lspci'], stdout=subprocess.PIPE)
+            p = subprocess.Popen(['lspci'], stdout=subprocess.PIPE, text=True)
             for line in p.stdout:
                 if "Audio" in line:
                     cards.append(line.split(":")[-1].strip())
             p.wait()
-            hardware_audio_card = ", ".join(cards)
-        except:
+            if cards != []:
+                hardware_audio_card = cards
+        except Exception:
             try:
                 hardware_audio_card = ""
                 cards = []
@@ -144,8 +195,9 @@ def get_system_info(as_string=False):
                     for line in f:
                         if line.startswith(" 0"):
                             cards.append(line.split(":")[1].strip())
-                hardware_audio_card = ", ".join(cards)
-            except:
+                if cards != []:
+                    hardware_audio_card = cards
+            except Exception:
                 hardware_audio_card = ""
 
         try:
@@ -155,20 +207,21 @@ def get_system_info(as_string=False):
             disk_free = int((s.f_frsize * s.f_bavail) // 1024 ** 2)
             hardware_disk_space_total = str(disk_total) + " MB"
             hardware_disk_space_free = str(disk_free) + " MB"
-        except:
+        except Exception:
             hardware_disk_space_total = ""
             hardware_disk_space_free = ""
 
         try:
             hardware_video_card = ""
             cards = []
-            p = subprocess.Popen(['lspci'], stdout=subprocess.PIPE)
+            p = subprocess.Popen(['lspci'], stdout=subprocess.PIPE, text=True)
             for line in p.stdout:
                 if "VGA" in line:
                     cards.append(line.split(":")[-1].strip())
             p.wait()
-            hardware_video_card = ", ".join(cards)
-        except:
+            if cards != []:
+                hardware_video_card = cards
+        except Exception:
             hardware_video_card = ""
 
     # Get platform specific info for Windows
@@ -178,13 +231,10 @@ def get_system_info(as_string=False):
         os_details = platform.win32_ver()[2]
         os_version = platform.win32_ver()[1]
 
-        try:
-            hardware_cpu_details = str(_get_registry_value(
-                "HKEY_LOCAL_MACHINE",
-                "HARDWARE\\DESCRIPTION\\System\\CentralProcessor\\0",
-                "ProcessorNameString"))
-        except:
-            hardware_cpu_details = ""
+        hardware_cpu_details = _get_registry_value(
+            "HKEY_LOCAL_MACHINE",
+            "HARDWARE\\DESCRIPTION\\System\\CentralProcessor\\0",
+            "ProcessorNameString")
 
         try:
             import ctypes
@@ -200,15 +250,15 @@ def get_system_info(as_string=False):
                             ("sullAvailExtendedVirtual", ctypes.c_ulonglong), ]
 
                 def __init__(self):
-                    # Initialize this to the size of MEMORYSTATUSEX
+                    # initialise this to the size of MEMORYSTATUSEX
                     self.dwLength = 2 * 4 + 7 * 8  # size = 2 ints, 7 longs
-                    return super(MEMORYSTATUSEX, self).__init__()
+                    super().__init__()
 
             stat = MEMORYSTATUSEX()
             ctypes.windll.kernel32.GlobalMemoryStatusEx(ctypes.byref(stat))
             hardware_memory_total = str(stat.ullTotalPhys // 1024 ** 2) + " MB"
             hardware_memory_free = str(stat.ullAvailPhys // 1024 ** 2) + " MB"
-        except:
+        except Exception:
             hardware_memory_total = ""
             hardware_memory_free = ""
 
@@ -219,67 +269,129 @@ def get_system_info(as_string=False):
             ret = ctypes.windll.kernel32.GetDiskFreeSpaceExW(
                 str(current_folder), ctypes.byref(_),
                 ctypes.byref(disk_total), ctypes.byref(disk_free))
-            hardware_disk_space_total = str(disk_total.value // 1024 ** 2) + " MB"
-            hardware_disk_space_free = str(disk_free.value // 1024 ** 2) + " MB"
-        except:
+            if ret:
+                hardware_disk_space_total = str(disk_total.value // 1024 ** 2) + " MB"
+                hardware_disk_space_free = str(disk_free.value // 1024 ** 2) + " MB"
+            else:
+                hardware_disk_space_total = ""
+                hardware_disk_space_free = ""
+        except Exception:
             hardware_disk_space_total = ""
             hardware_disk_space_free = ""
 
-        try:
-            tmp = str(_get_registry_value("HKEY_LOCAL_MACHINE",
-                                          "HARDWARE\\DEVICEMAP\\VIDEO",
-                                          "\Device\Video0"))
-            idx = tmp.find("System")
-            card = str(_get_registry_value("HKEY_LOCAL_MACHINE",
-                                           tmp[idx:].replace("\\", "\\\\"),
-                                           "Device Description"))
-            hardware_video_card = card
-        except:
+        video_cards = []
+        subkeys = _get_registry_subkeys(
+            "HKEY_LOCAL_MACHINE",
+            "SYSTEM\\CurrentControlSet\\Control\\Video")
+        for x in subkeys:
+            try:
+                found = _get_registry_value(
+                    "HKEY_LOCAL_MACHINE",
+                    f"SYSTEM\\CurrentControlSet\\Control\\Video\\{x}\\Video",
+                    "DeviceDesc")
+                video_cards.append(found.split(";")[-1])
+            except Exception:
+                pass
+
+        if video_cards != []:
+            hardware_video_card = video_cards
+        else:
             hardware_video_card = ""
 
-        hardware_audio_card = ""  # TODO
+        audio_cards = []
+        capture_subkeys = dict.fromkeys(_get_registry_subkeys(
+            "HKEY_LOCAL_MACHINE",
+            "SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\MMDevices\\Audio\\Capture"),
+                                        "Capture")
+        render_subkeys = dict.fromkeys(_get_registry_subkeys(
+            "HKEY_LOCAL_MACHINE",
+            "SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\MMDevices\\Audio\\Render"),
+                                       "Render")
 
-    # Get platform specific info for OS X
+        for x,y in {**capture_subkeys, **render_subkeys}.items():
+            try:
+                active = _get_registry_value(
+                    "HKEY_LOCAL_MACHINE",
+                    f"SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\MMDevices\\Audio\\{y}\\{x}",
+                    "DeviceState")
+                if active != "1":
+                    continue
+                found_description = _get_registry_value(
+                    "HKEY_LOCAL_MACHINE",
+                    f"SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\MMDevices\\Audio\\{y}\\{x}\\Properties",
+                    "{a45c254e-df1c-4efd-8020-67d146a850e0},2")
+                found_name = _get_registry_value(
+                    "HKEY_LOCAL_MACHINE",
+                    f"SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\MMDevices\\Audio\\{y}\\{x}\\Properties",
+                    "{b3f8fa53-0004-438e-9003-51a46e139bfc},6")
+
+                audio_cards.append(f"{found_description} ({found_name})")
+            except Exception:
+                pass
+
+        if audio_cards != []:
+            hardware_audio_card = audio_cards
+        else:
+            hardware_audio_card = ""
+
+
+    # Get platform specific info for MacOS
     elif sys.platform.startswith("darwin"):
         os_platform = "Darwin"
-        os_name = "Mac OS X"
+        os_name = "MacOS"
         os_details = platform.mac_ver()[1][1]
         os_version = platform.mac_ver()[0]
         try:
             proc = subprocess.Popen(['sysctl', '-a', 'hw.model'],
+                                    text=True,
                                     stdout=subprocess.PIPE,
                                     stdin=subprocess.PIPE)
             hardware_cpu_details = \
                     proc.stdout.readline().split(":")[1].strip()
-        except:
+        except Exception:
             hardware_cpu_details = ""
         try:
             proc = subprocess.Popen(['sysctl', '-a', 'hw.memsize'],
+                                    text=True,
                                     stdout=subprocess.PIPE,
                                     stdin=subprocess.PIPE)
             mem_total = int(proc.stdout.readline().split(":")[1].strip()) // 1024 ** 2
             hardware_memory_total = str(mem_total) + " MB"
-        except:
+        except Exception:
             hardware_memory_total = ""
 
         try:
             import re
             proc = subprocess.Popen(['vm_stat'],
+                                    text=True,
                                     stdout=subprocess.PIPE,
                                     stdin=subprocess.PIPE)
             output = proc.stdout.read().split("\n")
             for item in output:
-                x = item.find("Pages active:")
-                y = item.find("page size of")
-                if x > -1:
-                    non_decimal = re.compile(r'[^\d.]+')
-                    active = int(non_decimal.sub('', item).strip("."))
-                if y > -1:
+                a = item.find("page size of")
+                b = item.find("Anonymous pages:")
+                c = item.find("Pages purgeable:")
+                d = item.find("Pages wired down:")
+                e = item.find("Pages occupied by compressor:")
+                if a > -1:
                     non_decimal = re.compile(r'[^\d.]+')
                     page = int(non_decimal.sub('', item).strip("."))
-            hardware_memory_free = str(mem_total - (active * page) // 1024 ** 2) + " MB"
+                if b > -1:
+                    non_decimal = re.compile(r'[^\d.]+')
+                    anonymous = int(non_decimal.sub('', item).strip("."))
+                if c > -1:
+                    non_decimal = re.compile(r'[^\d.]+')
+                    purgeable = int(non_decimal.sub('', item).strip("."))
+                if d > -1:
+                    non_decimal = re.compile(r'[^\d.]+')
+                    wired = int(non_decimal.sub('', item).strip("."))
+                if e > -1:
+                    non_decimal = re.compile(r'[^\d.]+')
+                    compressed = int(non_decimal.sub('', item).strip("."))
 
-        except:
+            hardware_memory_free = str(mem_total - (((anonymous - purgeable) + wired + compressed) * page) // 1024 ** 2) + " MB"
+
+        except Exception:
             hardware_memory_free = ""
 
         try:
@@ -289,17 +401,23 @@ def get_system_info(as_string=False):
             disk_free = int((s.f_frsize * s.f_bavail) // 1024 ** 2)
             hardware_disk_space_total = str(disk_total) + " MB"
             hardware_disk_space_free = str(disk_free) + " MB"
-        except:
+        except Exception:
             hardware_disk_space_total = ""
             hardware_disk_space_free = ""
 
         try:
-            proc = subprocess.Popen(['system_profiler', 'SPAudioDataType'],
+            import plistlib
+            proc = subprocess.Popen(['system_profiler', 'SPAudioDataType',
+                                     '-xml'],
                                     stdout=subprocess.PIPE,
                                     stdin=subprocess.PIPE)
-            hardware_audio_card = \
-                proc.stdout.read().split("\n")[2].strip(":").strip()
-        except:
+            pl = plistlib.loads(proc.stdout.read())
+            hardware_audio_card = []
+            for x in pl[0]['_items']:
+                for card in x['_items']:
+                    hardware_audio_card.append(card['_name'])
+
+        except Exception:
             hardware_audio_card = ""
 
         try:
@@ -308,11 +426,11 @@ def get_system_info(as_string=False):
                                      '-xml'],
                                     stdout=subprocess.PIPE,
                                     stdin=subprocess.PIPE)
-            pl = plistlib.readPlist(proc.stdout)
+            pl = plistlib.loads(proc.stdout.read())
             hardware_video_card = []
             for card in pl[0]['_items']:
                 hardware_video_card.append(card['sppci_model'])
-        except:
+        except Exception:
             hardware_video_card = ""
 
     # Fill in info
@@ -323,9 +441,7 @@ def get_system_info(as_string=False):
     info["os_release"] = platform.release()
     info["os_version"] = os_version
     info["settings_folder"] = get_settings_folder()
-    info["python_expyriment_build_date"] = __date__
-    info["python_expyriment_revision"] = __revision__
-    info["python_expyriment_version"] = __version__
+    info["python_expyriment_version"] = get_version().split(" ")[0]
     if _numpy is not None:
         numpy_version = _numpy.version.version
     else:
@@ -372,10 +488,11 @@ def get_system_info(as_string=False):
     info["hardware_cpu_type"] = platform.processor()
     info["hardware_disk_space_free"] = hardware_disk_space_free
     info["hardware_disk_space_total"] = hardware_disk_space_total
+    info["hardware_displays"] = get_display_info()
     try:
         socket.gethostbyname("google.com")
         hardware_internet_connection = "Yes"
-    except:
+    except Exception:
         hardware_internet_connection = "No"
     info["hardware_internet_connection"] = hardware_internet_connection
     info["hardware_memory_total"] = hardware_memory_total
@@ -385,20 +502,23 @@ def get_system_info(as_string=False):
     info["hardware_ports_parallel_driver"] = ParallelPort.get_driver()
     try:
         info["hardware_ports_serial"] = [x.device for x in _com_port_list]
-    except:
+    except Exception:
         info["hardware_ports_serial"] = _com_port_list
     info["hardware_video_card"] = hardware_video_card
 
+    # sort output
+    info = dict(sorted(info.items()))
+
     if as_string:
-        sorted_keys = list(info.keys())
-        sorted_keys.sort()
+        longest = 0
+        for key in info.keys():
+            longest = max(len(key), longest)
+
         rtn = ""
-        for key in sorted_keys:
-            tabs = "\t" * (4 - int((len(key) + 1) // 8))
-            try:
-                rtn += key + ":" + tabs + info[key] + "\n"
-            except TypeError:
-                rtn += key + ":" + tabs + repr(info[key]) + "\n"
+        for key in info:
+            tabs = " " * (longest - len(key)) + "\t"
+            rtn += key + ":" + tabs + repr(info[key]) + "\n"
+
     else:
         rtn = info
 
