@@ -6,24 +6,25 @@ This module contains miscellaneous functions for expyriment.
 All classes in this module should be called directly via expyriment.misc.*:
 
 """
-from __future__ import absolute_import, print_function, division
-from builtins import *
 
 __author__ = 'Florian Krause <florian@expyriment.org>, \
 Oliver Lindemann <oliver@expyriment.org>'
-__version__ = ''
-__revision__ = ''
-__date__ = ''
 
-import os
-import sys
-import glob
-import random
 import colorsys
+import glob
+import math
+import os
+import random
+import sys
 
 import pygame
 
-from .._internals import PYTHON3, android, get_settings_folder, get_version
+try:
+    import pygame._sdl2.audio as sdl2_audio
+except ImportError:
+    sdl2_audio = None
+
+from .._internals import active_exp, get_settings_folder, get_version
 
 try:
     from locale import getdefaultlocale
@@ -39,14 +40,110 @@ if FS_ENC is None:
     FS_ENC = 'utf-8'
 
 
-_round = round
+class MediaTime(float):
+    """A class representing time as used in the context of media playback.
 
-def round(number, ndigits=0):
-    """Round half away from zero.
+    Internally represents time in seconds as a float value (inherits from
+    built-in float type), but can deal with multiple formats when creating and
+    comparing time.
+
+    Parameters
+    ----------
+    time : int, float, tuple, list or str
+        the time to represent; can be any of the following formats:
+            - float: seconds
+            - tuple: (minutes, seconds) or (hours, minutes, seconds)
+            - list: [minutes, seconds] or [hours, minutes, seconds]
+            - str: 'hh:mm:ss', 'hh:mm:ss.sss', or 'mm:ss.sss'
+
+    Examples
+    --------
+    >>> MediaTime(15.4)                # seconds
+    >>> MediaTime((1, 21.5))           # (min, sec)
+    >>> MediaTime((1, 1, 2))           # (hr, min, sec)
+    >>> MediaTime('01:01:33.5')        # (hr:min:sec)
+    >>> MediaTime('01:01:33.045')
+    >>> MediaTime('01:01:33,5')        # comma works too
+    >>> MediaTime(3600) == '01:00:00'  # True
+
+    """
+
+    @staticmethod
+    def convert_to_seconds(time):
+        """Try to convert time specified in common media formats into seconds.
+
+        Parameters
+        ----------
+        time : int, float, tuple, list or str
+            the time to convert; can be any of the following formats:
+                - float: seconds
+                - tuple: (minutes, seconds) or (hours, minutes, seconds)
+                - list: [minutes, seconds] or [hours, minutes, seconds]
+                - str: 'hhrmm:ss', 'hh:mm:ss.sss', or 'mm:ss.sss'
+
+        Returns
+        -------
+        float
+            The media time in seconds.
+
+        Examples
+        --------
+        >>> convert_to_seconds(15.4)          # seconds
+        15.4
+        >>> convert_to_seconds((1, 21.5))     # (min, sec)
+        81.5
+        >>> convert_to_seconds((1, 1, 2))     # (hr, min, sec)
+        3662
+        >>> convert_to_seconds('01:01:33.5')  # (hr:min:sec)
+        3693.5
+        >>> convert_to_seconds('01:01:33.045')
+        3693.045
+        >>> convert_to_seconds('01:01:33,5')  # comma works too
+        3693.5
+
+        """
+
+        # The following code has been adapted from MoviePy
+        # (https://zulko.github.io/moviepy/)
+
+        factors = (1, 60, 3600)
+
+        if isinstance(time, str):
+            time = [float(part.replace(",", ".")) for part in time.split(":")]
+
+        if not isinstance(time, (tuple, list)):
+            return time  # Return as is, if not a valid format
+
+        return sum(mult * part for mult, part in zip(factors, reversed(time)))
+
+        # End of code from MoviePy
+
+    def __new__(cls, time):
+        seconds = cls.convert_to_seconds(time)  # Use the static method
+        return super().__new__(cls, seconds)
+
+    def __lt__(self, other):
+        return float(self) < self.convert_to_seconds(other)
+
+    def __le__(self, other):
+        return float(self) <= self.convert_to_seconds(other)
+
+    def __eq__(self, other):
+        return float(self) == self.convert_to_seconds(other)
+
+    def __gt__(self, other):
+        return float(self) > self.convert_to_seconds(other)
+
+    def __ge__(self, other):
+        return float(self) >= self.convert_to_seconds(other)
+
+
+def py2_round(number, ndigits=0):
+    """Round half away from zero and always return a float.
 
     This method implements the Python 2 way of rounding.
-    For "bankers rounding" (round half to even), plese use the builtin `round`
-    function in Python 3 or Numpy's `around` function in Python 2.
+    For "bankers rounding" (round half to even), please use the builtin `round`
+    function or Numpy's `around` function.
 
     Parameters
     ----------
@@ -62,16 +159,11 @@ def round(number, ndigits=0):
 
     """
 
-    import math
-
-    if not PYTHON3:
-        return _round(number, ndigits)
+    p = 10 ** ndigits
+    if number > 0:
+        return float(math.floor((number * p) + 0.5))/p
     else:
-        p = 10 ** ndigits
-        if number > 0:
-            return float(math.floor((number * p) + 0.5))/p
-        else:
-            return float(math.ceil((number * p) - 0.5))/p
+        return float(math.ceil((number * p) - 0.5))/p
 
 
 def compare_codes(input_code, standard_codes, bitwise_comparison=True):
@@ -105,9 +197,8 @@ def compare_codes(input_code, standard_codes, bitwise_comparison=True):
             return False
 
 
-def byte2unicode(s, fse=False):
-    if (PYTHON3 and isinstance(s, str)) or\
-       (not PYTHON3 and isinstance(s, unicode)):
+def bytes_to_unicode(s, fse=False):
+    if isinstance(s, str):
         return s
 
     if fse:
@@ -123,7 +214,7 @@ def byte2unicode(s, fse=False):
     return u
 
 
-def unicode2byte(u, fse=False):
+def unicode_to_bytes(u, fse=False):
     if isinstance(u, bytes):
         return u
 
@@ -140,7 +231,7 @@ def unicode2byte(u, fse=False):
     return s
 
 
-def str2unicode(s, fse=False):
+def str_to_unicode(s, fse=False):
     """Convert str to unicode.
 
     Converts an input str or unicode object to a unicode object without
@@ -162,9 +253,10 @@ def str2unicode(s, fse=False):
     -------
     A unicode-type string.
     """
-    return byte2unicode(s, fse)
 
-def unicode2str(u, fse=False):
+    return bytes_to_unicode(s, fse)
+
+def unicode_to_str(u, fse=False):
     """Convert unicode to str.
 
     Converts an input str or unicode object to a str object without throwing
@@ -185,9 +277,9 @@ def unicode2str(u, fse=False):
     A str-type string.
     """
 
-    return unicode2byte(u, fse)
+    return unicode_to_bytes(u, fse)
 
-def numpad_digit_code2ascii(keycode):
+def numpad_digit_code_to_ascii(keycode):
     """Convert numpad keycode to the ascii code of that particular number
 
     If it is not a keypad digit code, no conversion takes place and the
@@ -223,8 +315,8 @@ def add_fonts(folder):
     # a while. By having a watchdog thread, we can check if this is the case
     # and notify the user accordingly.
 
-    import time
     import threading
+    import time
 
     def watchdog_timer(state):
         time.sleep(1)
@@ -242,7 +334,7 @@ def add_fonts(folder):
 
     for font in glob.glob(os.path.join(folder, "*")):
         if font[-4:].lower() in ['.ttf', '.ttc']:
-            font = byte2unicode(font, fse=True)
+            font = bytes_to_unicode(font, fse=True)
             name = os.path.split(font)[1]
             bold = name.find('Bold') >= 0
             italic = name.find('Italic') >= 0
@@ -310,23 +402,45 @@ def find_font(font):
         return ""
 
 
+def get_display_info():
+    """Return information about the available display(s).
+
+    Return
+    ------
+    info : dict of dicts
+        the information for each available display in the form
+             "maximal_resolution": ...,  # highest possible or "native", tuple
+             "desktop_resolution": ...}  # might differ (e.g. scaled), tuple
+
+    """
+
+    pygame.display.init()
+    info = {}
+    for x in range(pygame.display.get_num_displays()):
+        info[x] = {
+             "maximal_resolution": pygame.display.list_modes(display=x)[0],
+             "desktop_resolution": pygame.display.get_desktop_sizes()[x]}
+
+    return info
+
+
 def get_monitor_resolution():
     """Returns the monitor resolution
 
+    DEPRECATED! Use get_display_info instead.
+
     Returns
     -------
     resolution: (int, int)
         monitor resolution, screen resolution
 
     """
-
-    from .. import _internals
-    if _internals.active_exp.is_initialized:
-        return _internals.active_exp.screen.monitor_resolution
+    # will be remove with 1.1
+    pygame.display.init()
+    if active_exp.is_initialised:
+        return active_exp.screen.display_resolution
     else:
-        pygame.display.init()
-        return (pygame.display.Info().current_w,
-                pygame.display.Info().current_h)
+        return get_display_info[0]["maximal_resolution"]
 
 
 def is_ipython_running():
@@ -346,8 +460,8 @@ def is_idle_running():
 
 
 def is_interactive_mode():
-    """Returns if Python is running in interactive mode (such as IDLE or
-    IPthon)
+    """Return if Python is running in interactive mode (such as IDLE or
+    IPython)
 
     Returns
     -------
@@ -362,7 +476,7 @@ def is_interactive_mode():
 def is_android_running():
     """Return True if Exypriment runs on Android."""
 
-    return android is not None
+    return hasattr(sys, "getandroidapilevel")
 
 
 def has_internet_connection():
@@ -372,7 +486,7 @@ def has_internet_connection():
         host = socket.gethostbyname("google.com")
         socket.create_connection((host, 80), 2)
         return True
-    except:
+    except Exception:
         return False
 
 
@@ -437,7 +551,7 @@ def which(programme):
 def download_from_stash(content="all", branch=None):
     """Download content from the Expyriment stash.
 
-    Content will be stored in an Expyriment settings diretory (`.expyriment`
+    Content will be stored in an Expyriment settings directory (`.expyriment`
     or `~expyriment`, located in the current user's home directory).
 
     Parameters
@@ -454,20 +568,16 @@ def download_from_stash(content="all", branch=None):
 
     def show_progress(count, total, status=''):
         bar_len = 40
-        filled_len = int(round(bar_len * count / float(total)))
-        percents = round(100.0 * count / float(total), 1)
+        filled_len = int(py2_round(bar_len * count / float(total)))
+        percents = py2_round(100.0 * count / float(total), 1)
         bar = '=' * filled_len + ' ' * (bar_len - filled_len)
         sys.stdout.write('{:5.1f}% [{}] {}\r'.format(percents, bar, status))
         sys.stdout.flush()
 
-    if PYTHON3:
-        from urllib.request import urlopen, Request
-    else:
-        from urllib2 import urlopen
-
-    from tempfile import TemporaryFile
-    from zipfile import ZipFile
     from shutil import copyfileobj
+    from tempfile import TemporaryFile
+    from urllib.request import Request, urlopen
+    from zipfile import ZipFile
 
     if branch is None:
         branch = get_version().split(" ")[0]
@@ -476,16 +586,12 @@ def download_from_stash(content="all", branch=None):
     try:
         r = Request(url, headers={"Accept-Encoding": "gzip; deflate"})
         u = urlopen(r)
-    except:
+    except Exception:
         raise RuntimeError("Download of {0} failed!".format(url))
 
     with TemporaryFile() as f:
-        meta = u.info()
         try:
-            if PYTHON3:
-                file_size = int(u.getheader('Content-Length'))
-            else:
-                file_size = int(u.info().getheaders("Content-Length")[0])
+            file_size = int(u.getheader('Content-Length'))
             file_size_dl = 0
             block_sz = 8192
             while True:
@@ -497,7 +603,7 @@ def download_from_stash(content="all", branch=None):
                 show_progress(file_size_dl, file_size,
                               "downloading stash ({0})".format(branch))
             sys.stdout.write("\n")
-        except:
+        except Exception:
             show_progress(0, 100,
                           "downloading stash ({0})".format(branch))
             chunk = u.read()
@@ -507,8 +613,8 @@ def download_from_stash(content="all", branch=None):
             show_progress(100, 100,
                           "downloading stash ({0})".format(branch))
 
-        if get_settings_folder() is None:
-            os.makedirs(os.path.join(os.path.expanduser("~"), ".expyriment"))
+        if not os.path.isdir(get_settings_folder()):
+            os.makedirs(get_settings_folder())
         path = get_settings_folder()
 
         f_zip = ZipFile(f)
@@ -519,7 +625,7 @@ def download_from_stash(content="all", branch=None):
             content_ = ("examples", "extras", "tools")
         else:
             content_ = content
-        if not type(content_) in (list, tuple):
+        if type(content_) not in (list, tuple):
             content_ = (content_,)
         files = []
         root = f_zip.namelist()[0]
@@ -532,7 +638,7 @@ def download_from_stash(content="all", branch=None):
             if not filename:
                 try:
                     os.mkdir(os.path.join(path, os.path.relpath(member, root)))
-                except:
+                except Exception:
                     pass
                 files_installed += 1
                 continue
@@ -547,13 +653,14 @@ def download_from_stash(content="all", branch=None):
     print("")
 
 
-def py2py3_sort_array(array):
-    """Sorts an array with different types using the string representation
-    under Python2 and Python3. Sorts in place!
+def string_sort_array(array):
+    """Sorts an array with different types using the string representation.
+    Sorts in place!
 
     Returns
     -------
-    array: the sorted array
+    array : list
+        the sorted array
 
     """
 
@@ -563,6 +670,35 @@ def py2py3_sort_array(array):
 def _sorter_fnc(x):
     """sorter function for py_sort"""
     if x is None:
-        return str("")
+        return ""
     else:
         return str(x)
+
+def get_audio_devices(input_devices=False):
+    """Get the names of available audio devices.
+
+    Parameters
+    ----------
+    input_devices : bool, optional
+        return input (instead of output) device names (default=False)
+
+    Returns
+    -------
+    audio_devices : list
+        a list of audio device names
+
+    """
+
+    if sdl2_audio is None:
+        return None
+
+    is_initialised = pygame.mixer.get_init()
+    if not is_initialised:
+        pygame.mixer.init()
+
+    audio_devices = sdl2_audio.get_audio_device_names(input_devices)
+
+    if not is_initialised:
+        pygame.mixer.quit()
+
+    return audio_devices
