@@ -4,33 +4,32 @@ This module contains several classes and functions that help
 to handle, preprocessing and aggregate Expyriment data files.
 
 """
-from __future__ import absolute_import, print_function, division
-from builtins import *
 
 __author__ = 'Florian Krause <florian@expyriment.org>, \
 Oliver Lindemann <oliver@expyriment.org>'
-__version__ = ''
-__revision__ = ''
-__date__ = ''
 
 import os as _os
+
 try:
     import locale as _locale
 except ImportError:
     _locale = None  # Does not exist on Android
-import sys as _sys
-from types import ModuleType
-from glob import glob as _glob
-from copy import copy as _copy
 import codecs as _codecs
 import re as _re
+import sys as _sys
+from copy import copy as _copy
+from glob import glob as _glob
+from types import ModuleType
+
 try:
     import numpy as _np
-except:
+except Exception:
     _np = None
-from ...misc import unicode2byte as _unicode2str
-from ...misc import byte2unicode as _str2unicode
-from ...misc import py2py3_sort_array as _py2py3_sort_array
+
+from ... import __version__
+from ...misc._miscellaneous import bytes_to_unicode as _str_to_unicode
+from ...misc._miscellaneous import string_sort_array as _py2py3_sort_array
+from ...misc._miscellaneous import unicode_to_bytes as _unicode_to_str
 
 
 def read_datafile(filename, only_header_and_variable_names=False, encoding=None,
@@ -74,10 +73,10 @@ def read_datafile(filename, only_header_and_variable_names=False, encoding=None,
     if encoding is None:
         with open(filename, 'r') as fl:
             first_line = fl.readline()
-            encoding = _re.findall("coding[:=]\s*([-\w.]+)", first_line)
+            encoding = _re.findall(r"coding[:=]\s*([-\w.]+)", first_line)
             if encoding == []:
                 second_line = fl.readline()
-                encoding = _re.findall("coding[:=]\s*([-\w.]+)",
+                encoding = _re.findall(r"coding[:=]\s*([-\w.]+)",
                                        second_line)
                 if encoding == []:
                     encoding = [None]
@@ -88,17 +87,18 @@ def read_datafile(filename, only_header_and_variable_names=False, encoding=None,
     fl = _codecs.open(filename, 'rb', encoding[0], errors='replace')
     for ln in fl:
         # parse infos
-        ln = _str2unicode(ln.strip())
-        if not(ln.startswith("#")):
+        ln = _str_to_unicode(ln.strip())
+        if not (ln.startswith("#")):
             if variables is None:
                 variables = ln.split(delimiter)
                 if only_header_and_variable_names:
                     break
                 if read_variables is not None:
-                    read_in_columns = [variables.index(x) for x in read_variables]
+                    read_in_columns = [variables.index(x) for x in
+                                       read_variables]
                     variables = [variables[x] for x in read_in_columns]
             else:
-                row =ln.split(delimiter)
+                row = ln.split(delimiter)
                 if read_in_columns is not None:
                     row = [row[x] for x in read_in_columns]
                 data.append(row)
@@ -115,7 +115,7 @@ def read_datafile(filename, only_header_and_variable_names=False, encoding=None,
                 ln = ln.replace("#date:", "")
                 subject_info["date"] = ln.strip()
             else:
-                comments = comments + "\n" + ln
+                comments = f"{comments}\n{ln}"
     fl.close()
     # strip variables
     variables = [x.strip() for x in variables]
@@ -138,35 +138,37 @@ def write_csv_file(filename, data, varnames=None, delimiter=','):
 
     """
 
+    if len(_os.path.splitext(filename)[1]) == 0:
+        filename = f"{filename}.csv"
     _sys.stdout.write("write file: {0}".format(filename))
     try:
         _locale_enc = _locale.getdefaultlocale()[1]
-    except:
+    except Exception:
         _locale_enc = "UTF-8"
     with open(filename, 'wb') as f:
         header = "# -*- coding: {0} -*-\n".format(_locale_enc)
-        f.write(_unicode2str(header))
+        f.write(_unicode_to_str(header))
         if varnames is not None:
             for c, v in enumerate(varnames):
                 if c > 0:
-                    f.write(_unicode2str(delimiter))
-                f.write(_unicode2str(v))
-            f.write(_unicode2str("\n"))
+                    f.write(_unicode_to_str(delimiter))
+                f.write(_unicode_to_str(v))
+            f.write(_unicode_to_str("\n"))
         cnt = 0
         for row in data:
             for c, v in enumerate(row):
                 if c > 0:
-                    f.write(_unicode2str(delimiter))
-                f.write(_unicode2str(v))
+                    f.write(_unicode_to_str(delimiter))
+                f.write(_unicode_to_str(v))
                 cnt += 1
-            f.write(_unicode2str("\n"))
+            f.write(_unicode_to_str("\n"))
 
     print(" ({0} cells in {1} rows)".format(cnt, len(data)))
 
 
 def write_concatenated_data(data_folder, file_name, output_file=None,
                             delimiter=',', to_R_data_frame=False,
-                            names_comprise_glob_pattern =False):
+                            names_comprise_glob_pattern=False):
     """Concatenate data and write it to a csv file.
 
     All files that start with this name will be considered for the
@@ -199,15 +201,18 @@ def write_concatenated_data(data_folder, file_name, output_file=None,
 
     """
 
+    if output_file is None:
+        output_file = _os.path.splitext(file_name)[0] + ".csv"
+
     if to_R_data_frame:
         return Aggregator(data_folder=data_folder, file_name=file_name,
-                          names_comprise_glob_pattern=names_comprise_glob_pattern)\
-        .write_concatenated_data_to_R_data_frame(output_file=output_file)
+                          names_comprise_glob_pattern=names_comprise_glob_pattern) \
+            .write_concatenated_data_to_R_data_frame(output_file=output_file)
     else:
         return Aggregator(data_folder=data_folder, file_name=file_name,
-                          names_comprise_glob_pattern=names_comprise_glob_pattern)\
-        .write_concatenated_data(output_file=output_file, delimiter=delimiter)
-
+                          names_comprise_glob_pattern=names_comprise_glob_pattern) \
+            .write_concatenated_data(output_file=output_file,
+                                     delimiter=delimiter)
 
 
 def get_experiment_duration(event_filename):
@@ -228,19 +233,19 @@ def get_experiment_duration(event_filename):
 
     data, _, _, _ = read_datafile(event_filename)
 
-    start = end = None
+    start = stop = None
     for r in data:
         if r[1] == "Experiment":
-            if r[2]=="started":
+            if r[2] == "started":
                 start = int(r[0])
-            elif r[2]=="ended":
+            elif r[2] == "ended":
                 stop = int(r[0])
 
-    sec = (stop-start) / 1000.0
+    sec = (stop - start) / 1000.0
     return sec / 60.0
 
 
-class Aggregator(object):
+class Aggregator:
     """A class implementing a tool to aggregate Expyriment data.
 
     This class is used to handle the multiple data files of a Experiment
@@ -305,7 +310,7 @@ class Aggregator(object):
         """
 
         if not isinstance(_np, ModuleType):
-            message = """Aggregator can not be initialized.
+            message = """Aggregator can not be initialised.
 The Python package 'Numpy' is not installed."""
             raise ImportError(message)
 
@@ -323,25 +328,19 @@ The Python package 'Numpy' is not installed."""
 
     def __str__(self):
         """Getter for the current design as text string."""
-        design_str = "Data\n"
-        design_str = design_str + u"- file name: " + self._file_name + "\n"
-        design_str = design_str + u"- folder: " + self._data_folder + "\n"
-        design_str = design_str + u"- {0} subject_data sets\n".format(
-            len(self._data_files))
-        design_str = design_str + u"- {0} variables: {1}\n".format(
-            len(self.variables), self.variables)
-        design_str = design_str + u"- recoded variables: {0}\n".format(
-            self._recode_txt)
-        design_str = design_str + u"- computed variables: {0}\n".format(
-            self._computes_txt)
-        design_str = design_str + u"Design\n"
-        design_str = design_str + u"- independent Variables: {0}\n".format(
-            self._iv_txt)
-        design_str = design_str + u"- dependent Variables: {0}\n".format(
-            self._dv_txt)
-        design_str = design_str + u"- exclude: {0}\n".format(
-            self._exclusions_txt)
-        return design_str
+        return f"""\
+Data
+- file name: {self._file_name}
+- folder: {self._data_folder}
+- {len(self._data_files)} subject_data sets
+- {len(self.variables)} variables: {self.variables}
+- recoded variables: {self._recode_txt}
+- computed variables: {self._computes_txt}
+Design
+- independent Variables: {self._iv_txt}
+- dependent Variables: {self._dv_txt}
+- exclude: {self._exclusions_txt}
+"""
 
     def _parse_syntax(self, syntax, throw_exception):
         """Preprocess relation and operation syntax.
@@ -359,7 +358,7 @@ The Python package 'Numpy' is not installed."""
                 break
         if found is None:
             if throw_exception:
-                raise RuntimeError(u"Incorrect syntax: '{0}'".format(syntax))
+                raise RuntimeError("Incorrect syntax: '{0}'".format(syntax))
             else:
                 return None
         else:
@@ -372,7 +371,7 @@ The Python package 'Numpy' is not installed."""
             if variables == v:
                 return cnt
         if (throw_exception):
-            raise RuntimeError(u"Unknown variable name '{0}'".format(
+            raise RuntimeError("Unknown variable name '{0}'".format(
                 variables))
         return None
 
@@ -388,15 +387,15 @@ The Python package 'Numpy' is not installed."""
             dv_fnc = tmp[0].strip()
             try:
                 dv_txt = tmp[1].strip()
-            except:
+            except Exception:
                 raise RuntimeError(
-                    u"Incorrect syntax for DV: '{0}'".format(variable))
+                    "Incorrect syntax for DV: '{0}'".format(variable))
             var_id = self._get_variable_id(dv_txt, True)
             if dv_fnc in self._dv_functions:
                 self._dv.append([dv_fnc, var_id])
             else:
                 raise RuntimeError(
-                    u"Unknown function for dependent variable:" +
+                    "Unknown function for dependent variable:" +
                     " '{0}'".format(dv_fnc))
 
     def _add_compute_variable(self, compute_syntax):
@@ -410,15 +409,15 @@ The Python package 'Numpy' is not installed."""
             syntax = tmp[1].strip()
             syntax = syntax.replace("@@", "==")
             syntax = syntax.replace("##", "==")
-        except:
-            raise RuntimeError(u"Incorrect compute syntax: '{0}'".format(
+        except Exception:
+            raise RuntimeError("Incorrect compute syntax: '{0}'".format(
                 compute_syntax))
 
         variable_def = self._parse_syntax(syntax, throw_exception=True)
         if variable_def is None:
             variable_def = self._parse_operation(syntax, throw_exception=True)
         if self._get_variable_id(variable_name) is not None:
-            raise RuntimeError(u"Variable already defined '{0}'".format(
+            raise RuntimeError("Variable already defined '{0}'".format(
                 variable_name))
         else:
             self._variables.append(variable_name)
@@ -431,7 +430,7 @@ The Python package 'Numpy' is not installed."""
         if relation[1] in self._relations:
             self._exclusions.append(relation)
         else:
-            raise RuntimeError(u"Incorrect exclusion syntax: '{0}'".format(
+            raise RuntimeError("Incorrect exclusion syntax: '{0}'".format(
                 relation_syntax))
 
     def _add_variable_recoding(self, recode_syntax):
@@ -452,7 +451,7 @@ The Python package 'Numpy' is not installed."""
             error = True
 
         if error:
-            raise RuntimeError(u"Incorrect recoding syntax: '{0}'".format(
+            raise RuntimeError("Incorrect recoding syntax: '{0}'".format(
                 recode_syntax))
         else:
             self._recode.append([var_id, excl_array])
@@ -487,7 +486,7 @@ The Python package 'Numpy' is not installed."""
         # _add_exclusion
         try:
             col = _np.float64(data[:, column_id])
-        except:
+        except Exception:
             # handling strings
             col = data[:, column_id]
         try:
@@ -495,7 +494,7 @@ The Python package 'Numpy' is not installed."""
                 val = _np.float64(data[:, second_var_id])
             else:
                 val = _np.float64(value)
-        except:
+        except Exception:
             # handling strings
             if second_var_id is not None:
                 val = data[:, second_var_id]
@@ -510,27 +509,27 @@ The Python package 'Numpy' is not installed."""
             mean_stds = self._dv_mean_std(data, column_id)
             idx = []
             if relation not in [">", "<", "=>", ">=", "=<", "<="]:
-                raise RuntimeError(u"Incorrect syntax for " +
-                                   u"exception: '{0} {1}'".format(
+                raise RuntimeError("Incorrect syntax for " +
+                                   "exception: '{0} {1}'".format(
                                        relation, value))
             for cnt, row in enumerate(data):
-                #find name of combination
+                # find name of combination
                 combi_str = self.variables[column_id]
                 for iv in self._iv:
                     _row_data = row[iv]
                     combi_str = combi_str + "_" + \
-                        u"{0}{1}".format(self.variables[iv],
-                                        _row_data)
+                                "{0}{1}".format(self.variables[iv],
+                                                 _row_data)
                 deviation = float(row[column_id]) - mean_stds[combi_str][0]
                 if (relation == ">" and
                     deviation > fac * mean_stds[combi_str][1]) or \
-                   (relation == "=>" or relation == ">=" and
-                    deviation >= fac * mean_stds[combi_str][1]) or \
-                   (relation == "<" and
-                    deviation < -fac * mean_stds[combi_str][1]) or \
-                   (relation == "=<" or relation == "<=" and
-                        deviation <= -fac * mean_stds[combi_str][1]):
-                        idx.append(cnt)
+                        (relation == "=>" or relation == ">=" and
+                         deviation >= fac * mean_stds[combi_str][1]) or \
+                        (relation == "<" and
+                         deviation < -fac * mean_stds[combi_str][1]) or \
+                        (relation == "=<" or relation == "<=" and
+                         deviation <= -fac * mean_stds[combi_str][1]):
+                    idx.append(cnt)
             return idx
         else:
             if relation == "!=":
@@ -549,7 +548,7 @@ The Python package 'Numpy' is not installed."""
                 comp = None  # should never occur
             if isinstance(comp, bool):
                 raise RuntimeError(
-                    u"Incorrect syntax for " + u"exception: '{0} {1}'".format(
+                    "Incorrect syntax for " + "exception: '{0} {1}'".format(
                         relation, value))
             return _np.flatnonzero(comp)
 
@@ -580,10 +579,10 @@ The Python package 'Numpy' is not installed."""
                         idx = idx & tmp
             # calc std over idx
             if len(idx) > 0:
-                result[new_variable_names[cnt+1]] = [
+                result[new_variable_names[cnt + 1]] = [
                     _np.mean(_np.float64(data[idx, column_dv_id])),
                     _np.std(_np.float64(data[idx, column_dv_id]))]
-                    # ignore first new var name, which is subject_id
+                # ignore first new var name, which is subject_id
         return result
 
     def _get_new_variables(self, iv_values):
@@ -608,9 +607,11 @@ The Python package 'Numpy' is not installed."""
                 if pos <= 0:  # end reached
                     return None
                 else:
-                    for x in range(pos, len(comb)):  # set to zero & all pos. behind
+                    for x in range(pos,
+                                   len(comb)):  # set to zero & all pos. behind
                         comb[x] = 0
-                    return increase_combination(comb, maxima, pos - 1)  # increase position before
+                    return increase_combination(comb, maxima,
+                                                pos - 1)  # increase position before
             else:
                 return comb
 
@@ -630,16 +631,16 @@ The Python package 'Numpy' is not installed."""
                 for c, x in enumerate(tmp_comb):
                     comb_values.append(iv_values[c][x])
                     if len(txt) > 0:
-                        txt = txt + "_"
-                    txt = txt + u"{0}{1}".format(self.variables[self._iv[c]],
-                                comb_values[-1])
+                        txt += "_"
+                    txt += "{0}{1}".format(self.variables[self._iv[c]],
+                                           comb_values[-1])
                 names.append(txt)
                 factor_combinations.append(comb_values)
                 tmp_comb = increase_combination(tmp_comb, n_levels)
 
         new_variable_names = ["subject_id"]
         for sv in self.subject_variables:
-            new_variable_names.append(u"{0}".format(sv))
+            new_variable_names.append("{0}".format(sv))
 
         for dv in self._dv:
             if dv[0] == "n_trials":
@@ -648,9 +649,9 @@ The Python package 'Numpy' is not installed."""
                 dv_txt = self.variables[dv[1]]
             if len(names) > 0:
                 for n in names:
-                    new_variable_names.append(u"{0}_{1}".format(dv_txt, n))
+                    new_variable_names.append("{0}_{1}".format(dv_txt, n))
             else:
-                new_variable_names.append(u"{0}_total".format(dv_txt))
+                new_variable_names.append("{0}_total".format(dv_txt))
 
         return new_variable_names, factor_combinations
 
@@ -707,26 +708,25 @@ The Python package 'Numpy' is not installed."""
                     files.append(_os.path.join(data_folder, flname))
 
         for flname in files:
-                _data, vnames, _subject_info, _comments = \
-                    read_datafile(flname, read_variables=variables)
-                if len(self._variables) < 1:
-                    self._variables = vnames
-                else:
-                    if vnames != self._variables:
-                        message = u"Different variables in ".format(flname)
-                        message = message + u"\n{0}".format(vnames)
-                        message = message + u"\ninstead of\n{0}".format(
-                            self._variables)
-                        raise RuntimeError(message)
-                self._data_files.append(flname)
+            _data, vnames, _subject_info, _comments = \
+                read_datafile(flname, read_variables=variables)
+            if len(self._variables) < 1:
+                self._variables = vnames
+            else:
+                if vnames != self._variables:
+                    message = (f"Different variables in {flname}"
+                               f"\n{vnames}"
+                               f"\ninstead of\n{self._variables}")
+                    raise RuntimeError(message)
+            self._data_files.append(flname)
 
         if len(self._data_files) < 1:
-            raise Exception(u"No data files found")
+            raise Exception("No data files found")
 
-        print(u"found {0} subject_data sets".format(len(self._data_files)))
-        print(u"found {0} variables: {1}".format(len(self._variables),
-                                                [x for x in self._variables]))
 
+        print("found {0} subject_data sets".format(len(self._data_files)))
+        print("found {0} variables: {1}".format(len(self._variables),
+                                                 list(self._variables)))
 
     @property
     def data_files(self):
@@ -796,7 +796,7 @@ The Python package 'Numpy' is not installed."""
 
     def get_data(self, filename, recode_variables=True,
                  compute_new_variables=True, exclude_trials=True):
-        """Read data from from a single Expyriment data file.
+        """Read data from a single Expyriment data file.
 
         Notes
         -----
@@ -834,12 +834,12 @@ The Python package 'Numpy' is not installed."""
 
         # check filename
         if filename not in self._data_files:
-            raise RuntimeError(u"'{0}' is not in the data list\n".format(
+            raise RuntimeError("'{0}' is not in the data list\n".format(
                 filename))
 
         data, _vnames, subject_info, comments = \
             read_datafile(filename)
-        print(u"   reading {0}".format(filename))
+        print("   reading {0}".format(filename))
 
         if recode_variables:
             for var_id, recoding in self._recode:
@@ -869,7 +869,7 @@ The Python package 'Numpy' is not installed."""
                                 [data[:, second_var_id]]).transpose()
                         else:
                             b = _np.float64(var_def[2])
-                    except:
+                    except Exception:
                         msg = "Error while computing new variable {0}. " + \
                               "Non-number in variables of {1}"
                         msg.format(new_var_name, filename)
@@ -890,7 +890,7 @@ The Python package 'Numpy' is not installed."""
         for sv in self.subject_variables:
             try:
                 info = subject_info[sv]
-            except:
+            except Exception:
                 info = "nan"
             col = _np.array([[info for _x in range(data.shape[0])]])
             data = _np.c_[data, col.transpose()]
@@ -973,7 +973,7 @@ The Python package 'Numpy' is not installed."""
         data = self.concatenated_data[0]
         try:
             data = _np.float64(data[:, cols])
-        except:
+        except Exception:
             data = data[:, cols]
 
         return data
@@ -1034,7 +1034,6 @@ The Python package 'Numpy' is not installed."""
         write_csv_file(filename=output_file, data=data[0], varnames=data[1],
                        delimiter=delimiter)
 
-
     def set_independent_variables(self, variables):
         """Set the independent variables.
 
@@ -1215,7 +1214,8 @@ The Python package 'Numpy' is not installed."""
 
         self._computes = []
         self._variables = read_datafile(self._data_files[0],
-                                        only_header_and_variable_names=True)[1]  # original variables
+                                        only_header_and_variable_names=True)[
+            1]  # original variables
         for syntax in self._computes_txt:
             self._add_compute_variable(syntax)
         self._last_data = []
@@ -1247,7 +1247,7 @@ The Python package 'Numpy' is not installed."""
             for cnt, var in enumerate(varnames):
                 if cnt > 0:
                     _row_data = row[cnt]
-                    print(u"\t{0}:\t{1}".format(var[4:], _row_data))
+                    print("\t{0}:\t{1}".format(var[4:], _row_data))
         print("\n")
         self._dv = old_dv
         self._iv = old_iv
@@ -1274,8 +1274,7 @@ The Python package 'Numpy' is not installed."""
         """
 
         data, _variables = self.concatenated_data
-        subjects = list(set(data[:, column_subject_id]))
-        subjects.sort()
+        subjects = sorted(set(data[:, column_subject_id]))
         # get all iv values
         iv_values = []
         for iv in self._iv:
@@ -1321,9 +1320,9 @@ The Python package 'Numpy' is not installed."""
                         elif dv[0] == "n_trials":
                             row.append(values.shape[0])
                         else:
-                            row.append(_np.NaN)
+                            row.append(_np.nan)
                     else:
-                        row.append(_np.NaN)
+                        row.append(_np.nan)
             if result is None:
                 result = _np.array([row], dtype='|U99')
             else:
