@@ -1,19 +1,14 @@
-"""
-The misc extra package.
+"""The misc extra package.
 
 Notes
 -----
-    To us the extras module you have to import it manually by calling:
-    `import expyriment.misc.extras`
+To us the extras module you have to import it manually by calling:
+`import expyriment.misc.extras`
 
 """
 
 __author__ = 'Florian Krause <florian@expyriment.org> \
 Oliver Lindemann <oliver@expyriment.org>'
-__version__ = ''
-__revision__ = ''
-__date__ = ''
-
 
 from ... import _internals
 
@@ -28,5 +23,5 @@ for name, code in _internals.import_plugins_code("misc").items():
 
 try:
     del (name, code)
-except:
-    pass
\ No newline at end of file
+except Exception:
+    pass
