@@ -1,23 +1,16 @@
-#!/usr/bin/env python
-
 """
 A blank screen stimulus.
 
 This module contains a class implementing a blank screen stimulus.
 
 """
-from __future__ import absolute_import, print_function, division
-from builtins import *
 
 __author__ = 'Florian Krause <florian@expyriment.org>, \
 Oliver Lindemann <oliver@expyriment.org>'
-__version__ = ''
-__revision__ = ''
-__date__ = ''
 
-
-from ._canvas import Canvas
 from .. import _internals
+from ._canvas import Canvas
+
 
 class BlankScreen(Canvas):
     """A class implementing a blank screen."""
@@ -37,7 +30,7 @@ class BlankScreen(Canvas):
             self._colour = _internals.active_exp.background_colour
         try:
             size = _internals.active_exp.screen.surface.get_size()
-        except:
+        except Exception:
             raise RuntimeError("Could not get size of screen!")
         Canvas.__init__(self, size, colour=self._colour)
 
@@ -48,7 +41,7 @@ class BlankScreen(Canvas):
             from .. import control
             control.set_develop_mode(True)
             control.defaults.event_logging.event_logging = 0
-            exp_ = control.initialize()
+            exp_ = control.initialise()
         blankscreen = BlankScreen()
         blankscreen.present()
         if exp is None:
