@@ -1,30 +1,22 @@
-#!/usr/bin/env python
-
 """
 A Shape stimulus.
 
 This module contains a class implementing a shape stimulus.
 
 """
-from __future__ import absolute_import, print_function, division
-from builtins import *
-
 
 __author__ = 'Florian Krause <florian@expyriment.org>, \
 Oliver Lindemann <oliver@expyriment.org>'
-__version__ = ''
-__revision__ = ''
-__date__ = ''
 
 import copy
-from math import sqrt
+
 import pygame
 
-from . import defaults
-from ._visual import Visual
 from .. import _internals
 from ..misc._timer import get_time
-from ..misc.geometry import XYPoint, lines_intersect, position2coordinates
+from ..misc.geometry import XYPoint, lines_intersect
+from . import defaults
+from ._visual import Visual
 
 
 def _get_shape_rect(points):
@@ -52,7 +44,6 @@ class Shape(Visual):
     """A class implementing a shape."""
 
     def __init__(self, position=None, colour=None,
-                 line_width=None,
                  anti_aliasing=None,
                  vertex_list=None,
                  debug_contour_colour = None
@@ -62,7 +53,7 @@ class Shape(Visual):
 
         A shape is an object described by vertices. For more details about
         vertex representations see:
-        http://en.wikipedia.org/wiki/Vertex_(geometry)
+        https://en.wikipedia.org/wiki/Vertex_(geometry)
 
         The vertex representation describes the contour of an object. Think of
         it as if you would draw with a pen. You start somewhere and make
@@ -131,7 +122,7 @@ class Shape(Visual):
             list of vertices (int, int)
         debug_contour_colour : (int, int, int), optional
             The colour of the contour of the shape.
-            If None (default), contour colour is not displyed.
+            If None (default), contour colour is not displayed.
             Use this option only for debugging. The resulting surface size
             might be enlarged by one pixel large depending on the shape.
 
@@ -242,13 +233,13 @@ class Shape(Visual):
 
     @property
     def debug_contour_colour(self):
-        """Getter for conture_colour."""
+        """Getter for contour_colour."""
 
         return self._debug_contour_colour
 
     @debug_contour_colour.setter
     def debug_contour_colour(self, colour):
-        """Setter for conture_colour."""
+        """Setter for contour_colour."""
 
         if self.has_surface:
             raise AttributeError(Shape._getter_exception_message.format(
@@ -274,7 +265,7 @@ class Shape(Visual):
 
         Notes
         -----
-        Fom version 0.9.1 on, this is a pygame.Rect
+        From version 0.9.1 on, this is a pygame.Rect
 
         """
 
@@ -312,7 +303,7 @@ class Shape(Visual):
 
     @property
     def points_on_screen(self):
-        """Returns shape as list of tuples (x, y) resenting points on the
+        """Returns shape as list of tuples (x, y) representing points on the
         screen. see Shape.xy_points_on_screen.
 
         In contrast to Shape.points, it takes into account the position on
@@ -411,7 +402,7 @@ class Shape(Visual):
         Parameters
         ----------
         vertex_list : ((int, int))
-            list of vertices ((int, int))
+            list of vertices as tuples((int, int))
 
         """
         type_error_message = "The method add_vertices requires a list of" + \
@@ -484,11 +475,6 @@ class Shape(Visual):
         pt = XYPoint(position)
         return pt.is_inside_polygon(self.xy_points_on_screen)
 
-    def is_point_inside(self, point_xy):
-        """"OBSOLETE METHOD: Please use 'overlapping_with_position'."""
-
-        raise DeprecationWarning("is_point_inside is an obsolete method. Please use overlapping_with_position")
-
     def overlapping_with_shape(self, other):
         """Return true if shape overlaps with other shape.
 
@@ -504,8 +490,8 @@ class Shape(Visual):
 
         """
 
-        # shape and other shape do not overlap if 
-        # (a) no point of shape is inside other shape 
+        # shape and other shape do not overlap if
+        # (a) no point of shape is inside other shape
         # (b) AND no point of other shape is inside shape
         # (c) AND lines do not intersect
 
@@ -527,11 +513,6 @@ class Shape(Visual):
                             return True
         return False
 
-    def is_shape_overlapping(self, shape2):
-        """OBSOLETE METHOD: Please use 'overlapping_with_shape'."""
-
-        raise DeprecationWarning("is_shape_overlapping is an obsolete method. Please use overlapping_with_shape.")
-
     def native_rotate(self, degree):
         """Rotate the shape.
 
@@ -614,7 +595,7 @@ class Shape(Visual):
         Parameters
         ----------
         level : int
-            level of bluring
+            level of blurring
 
         Returns
         -------
@@ -711,7 +692,7 @@ class Shape(Visual):
             from .. import control
             control.set_develop_mode(True)
             control.defaults.event_logging = 0
-            exp_ = control.initialize()
+            exp_ = control.initialise()
         sh = Shape(position=(10, 100), colour=(255, 0, 255))
         sh.add_vertices([(60, 60), (0, -120)])
         sh.present()
