@@ -1,27 +1,22 @@
 """
 This module contains the base classes for visual stimuli.
 """
-from __future__ import absolute_import, print_function, division
-from builtins import *
-
 
 __author__ = 'Florian Krause <florian@expyriment.org>, \
 Oliver Lindemann <oliver@expyriment.org>'
-__version__ = ''
-__revision__ = ''
-__date__ = ''
 
-
-import tempfile
-import os
 import copy
-import random
 import itertools
+import os
+import random
+import tempfile
+from abc import ABC
 
 import pygame
+
 try:
-    import OpenGL.GLU as oglu
     import OpenGL.GL as ogl
+    import OpenGL.GLU as oglu
 except ImportError:
     oglu = None
     ogl = None
@@ -30,17 +25,16 @@ try:
 except ImportError:
     np = None
 
-from . import defaults
-from .. import _internals
-from ._stimulus import Stimulus
-from .. import misc
-from ..misc import geometry, unicode2byte
+from .. import _internals, misc
+from ..misc import geometry, unicode_to_bytes
 from ..misc._timer import get_time
+from . import defaults
+from ._stimulus import Stimulus
 
 random.seed()
 
 
-class _LaminaPanelSurface(object):
+class _LaminaPanelSurface:
     """A class implementing an OpenGL surface."""
 
     # The following code is based on part of the Lamina module by David Keeney
@@ -48,7 +42,7 @@ class _LaminaPanelSurface(object):
     # into expyriment (e.g. positioning)
     def __init__(self, surface, quadDims=(-1, 1, 1, 1),
                  position=(0, 0)):
-        """Initialize new instance.
+        """initialise new instance.
 
         Parameters
         ----------
@@ -91,7 +85,7 @@ class _LaminaPanelSurface(object):
         if getattr(self, '_txtr', None) is not None:
             try:
                 ogl.glDeleteTextures([self._txtr])
-            except:
+            except Exception:
                 pass
 
     def convertMousePos(self, pos):
@@ -114,7 +108,7 @@ class _LaminaPanelSurface(object):
 
         screensize = pygame.display.get_surface().get_size()
         bottomleft = oglu.gluUnProject(screensize[0] // 2 - \
-                                       int(misc.round(self._winsize[0] / 2)) + \
+                                       int(misc.py2_round(self._winsize[0] / 2)) + \
                                        self._position[0],
                                        screensize[1] // 2 - \
                                        self._winsize[1] // 2 + \
@@ -126,16 +120,16 @@ class _LaminaPanelSurface(object):
                                         self._winsize[1] // 2 + \
                                         self._position[1], 0)
         topleft = oglu.gluUnProject(screensize[0] // 2 - \
-                                    int(misc.round(self._winsize[0] / 2)) + \
+                                    int(misc.py2_round(self._winsize[0] / 2)) + \
                                     self._position[0],
                                     screensize[1] // 2 + \
-                                    int(misc.round(self._winsize[1] / 2)) + \
+                                    int(misc.py2_round(self._winsize[1] / 2)) + \
                                     self._position[1], 0)
         topright = oglu.gluUnProject(screensize[0] // 2 + \
                                      self._winsize[0] // 2 + \
                                      self._position[0],
                                      screensize[1] // 2 + \
-                                     int(misc.round(self._winsize[1] / 2)) + \
+                                     int(misc.py2_round(self._winsize[1] / 2)) + \
                                      self._position[1], 0)
 
 
@@ -170,7 +164,7 @@ class _LaminaPanelSurface(object):
     # End of code based on Lamina module
 
 
-class Visual(Stimulus):
+class Visual(Stimulus, ABC):
     """A class implementing a general visual stimulus.
 
     All other visual stimuli should be subclassed from this class since it
@@ -203,6 +197,7 @@ class Visual(Stimulus):
             textureData = pygame.image.tostring(surf, "RGBA", 1)
             colours = ogl.GL_RGBA
             width, height = surf.get_size()
+            ogl.glPixelStorei(ogl.GL_UNPACK_ALIGNMENT, 4)
         else:
             textureData = surf
             if textureData.shape[2] == 3:
@@ -210,6 +205,7 @@ class Visual(Stimulus):
             elif textureData.shape[2] == 4:
                 colours = ogl.GL_RGBA
             width, height = len(surf[0]), len(surf)
+            ogl.glPixelStorei(ogl.GL_UNPACK_ALIGNMENT, 1)
         ogl.glEnable(ogl.GL_TEXTURE_2D)
         ogl.glBindTexture(ogl.GL_TEXTURE_2D, txtr)
         ogl.glTexImage2D(ogl.GL_TEXTURE_2D, 0, colours, width, height, 0,
@@ -250,7 +246,7 @@ class Visual(Stimulus):
 
         self._was_compressed_before_preload = None
 
-        if not _internals.active_exp.is_initialized:
+        if not _internals.active_exp.is_initialised:
             warn_message = "Stimulus created before initializing " + \
                            "(experiment defaults won't apply)!"
             print("Warning: " + warn_message)
@@ -259,18 +255,18 @@ class Visual(Stimulus):
                                      "or compressed stimulus!"
 
     def __del__(self):
-        """ Clear surface and ogl_screen when when the objects is deconstructed.
+        """ Clear surface and ogl_screen when the objects is deconstructed.
 
         """
 
         try:
             self.clear_surface()
-        except:
+        except Exception:
             pass
         if self._compression_filename is not None:
             try:
                 os.remove(self._compression_filename)
-            except:
+            except Exception:
                 pass
 
     @property
@@ -293,7 +289,7 @@ class Visual(Stimulus):
     def polar_position(self):
         """Getter for the position in polar coordinates (radial, angle[degrees])"""
 
-        return geometry.cartesian2polar(self._position)
+        return geometry.cartesian_to_polar(self._position)
 
     @polar_position.setter
     def polar_position(self, value):
@@ -303,8 +299,8 @@ class Visual(Stimulus):
 
         """
 
-        pos = geometry.polar2cartesian(value)
-        self.reposition((int(misc.round(pos[0])), int(misc.round(pos[1]))))
+        pos = geometry.polar_to_cartesian(value)
+        self.reposition((int(misc.py2_round(pos[0])), int(misc.py2_round(pos[1]))))
 
     @property
     def absolute_position(self):
@@ -313,7 +309,7 @@ class Visual(Stimulus):
         Notes
         -----
         The absolute position differs for instance from the (relative) position, if the
-        stimulus is plotted ontop of another stimulus, which has not the position (0,0).
+        stimulus is plotted on top of another stimulus, which has not the position (0,0).
 
         """
 
@@ -463,8 +459,7 @@ class Visual(Stimulus):
 
         """
 
-        surface = pygame.surface.Surface((0, 0))
-        return surface
+        return pygame.surface.Surface((0, 0))
 
     def _set_surface(self, surface):
         """Set the surface (from internal use only).
@@ -509,25 +504,28 @@ class Visual(Stimulus):
 
         """
 
-        if self.has_surface:
+        has_surface = self.has_surface
+        is_preloaded = self.is_preloaded
+        is_compressed = self.is_compressed
+        if has_surface:
             surface_backup = self._get_surface()
             surface_copy = self._get_surface().copy()
             self._surface = None
         rtn = Stimulus.copy(self)
-        if self.has_surface:
+        if has_surface:
             self._surface = surface_backup
             rtn._surface = surface_copy
             rtn._is_preloaded = False
             rtn._ogl_screen = None
             rtn._is_compressed = False
             rtn._compression_filename = None
-        if self.is_preloaded:
-            if _internals.active_exp.screen.open_gl:
+        if is_preloaded:
+            if _internals.active_exp.screen.opengl:
                 self._ogl_screen = _LaminaPanelSurface(
                     self._get_surface(),
                     position=self.position)
             rtn.preload()
-        if self.is_compressed:
+        if is_compressed:
             rtn.compress()
         rtn._was_compressed_before_preload = \
                 self._was_compressed_before_preload
@@ -554,9 +552,6 @@ class Visual(Stimulus):
         return geometry.XYPoint(
             self.position).distance(geometry.XYPoint(other.position))
 
-    def replace(self, dummy):
-        raise DeprecationWarning("Replace is an obsolete method. Please use reposition!")
-
     def reposition(self, new_position):
         """Move stimulus to a new position.
 
@@ -640,7 +635,7 @@ class Visual(Stimulus):
             screen_size = _internals.active_exp.screen.surface.get_size()
             self_size = self.surface_size
             other_size = stimulus.surface_size
-            self_pos = geometry.position2coordinates(self.position,
+            self_pos = geometry.position_to_coordinates(self.position,
                                                      screen_size)
             self_pos[0] -= self_size[0] // 2
             self_pos[1] -= self_size[1] // 2
@@ -648,7 +643,7 @@ class Visual(Stimulus):
                 self_pos[0] += 1
             if self_size[1] % 2 == 0:
                 self_pos[1] += 1
-            other_pos = geometry.position2coordinates(stimulus.position,
+            other_pos = geometry.position_to_coordinates(stimulus.position,
                                                       screen_size)
             other_pos[0] -= other_size[0] // 2
             other_pos[1] -= other_size[1] // 2
@@ -660,14 +655,11 @@ class Visual(Stimulus):
             self_mask = pygame.mask.from_surface(self._get_surface())
             other_mask = pygame.mask.from_surface(stimulus._get_surface())
             overlap = self_mask.overlap_area(other_mask, offset)
-            if overlap > 0 and overlap == self_mask.count():
-                return True
-            else:
-                return False
+            return overlap > 0 and overlap == self_mask.count()
 
         elif mode == "surface":
             screen_size = _internals.active_exp.screen.surface.get_size()
-            sx, sy = geometry.coordinates2position(self.absolute_position,
+            sx, sy = geometry.coordinates_to_position(self.absolute_position,
                                                    screen_size)
             selfrect = pygame.Rect((0, 0), self.surface_size)
             if self.surface_size[0] % 2 == 0:
@@ -675,20 +667,17 @@ class Visual(Stimulus):
             if self.surface_size[1] % 2 == 0:
                 sy += 1
             selfrect.center = (sx, sy)
-            ox, oy = geometry.coordinates2position(stimulus.absolute_position,
+            ox, oy = geometry.coordinates_to_position(stimulus.absolute_position,
                                                    screen_size)
             if stimulus.surface_size[0] % 2 == 0:
                 ox += 1
             if stimulus.surface_size[1] % 2 == 0:
                 oy += 1
             stimrect = pygame.Rect((0, 0), stimulus.surface_size)
-            stimrect.right = stimrect.right + 1
-            stimrect.bottom = stimrect.bottom + 1
+            stimrect.right += 1
+            stimrect.bottom += 1
             stimrect.center = (ox, oy)
-            if stimrect.contains(selfrect):
-                return True
-            else:
-                return False
+            return stimrect.contains(selfrect)
 
     def overlapping_with_stimulus(self, stimulus, mode="visible",
                                   use_absolute_position=True):
@@ -730,7 +719,7 @@ class Visual(Stimulus):
             self_size = self.surface_size
             other_size = stimulus.surface_size
             if use_absolute_position:
-                self_pos = geometry.position2coordinates(
+                self_pos = geometry.position_to_coordinates(
                     self.absolute_position, screen_size)
                 self_pos[0] -= self_size[0] // 2
                 self_pos[1] -= self_size[1] // 2
@@ -739,7 +728,7 @@ class Visual(Stimulus):
                 if self_size[1] % 2 == 0:
                     self_pos[1] += 1
 
-                other_pos = geometry.position2coordinates(
+                other_pos = geometry.position_to_coordinates(
                     stimulus.absolute_position, screen_size)
                 other_pos[0] -= other_size[0] // 2
                 other_pos[1] -= other_size[1] // 2
@@ -749,7 +738,7 @@ class Visual(Stimulus):
                     other_pos[1] += 1
 
             else:
-                self_pos = geometry.position2coordinates(
+                self_pos = geometry.position_to_coordinates(
                     self.position, screen_size)
                 self_pos[0] -= self_size[0] // 2
                 self_pos[1] -= self_size[1] // 2
@@ -758,7 +747,7 @@ class Visual(Stimulus):
                 if self_size[1] % 2 == 0:
                     self_pos[1] += 1
 
-                other_pos = geometry.position2coordinates(
+                other_pos = geometry.position_to_coordinates(
                     stimulus.position, screen_size)
                 other_pos[0] -= other_size[0] // 2
                 other_pos[1] -= other_size[1] // 2
@@ -779,14 +768,14 @@ class Visual(Stimulus):
         elif mode == "surface":
             screen_size = _internals.active_exp.screen.surface.get_size()
             if use_absolute_position:
-                sx, sy = geometry.coordinates2position(
+                sx, sy = geometry.coordinates_to_position(
                     self.absolute_position, screen_size)
-                ox, oy = geometry.coordinates2position(
+                ox, oy = geometry.coordinates_to_position(
                     stimulus.absolute_position, screen_size)
             else:
-                sx, sy = geometry.coordinates2position(
+                sx, sy = geometry.coordinates_to_position(
                     self.position, screen_size)
-                ox, oy = geometry.coordinates2position(
+                ox, oy = geometry.coordinates_to_position(
                     stimulus.position, screen_size)
             selfrect = pygame.Rect((0, 0), self.surface_size)
             if self.surface_size[0] % 2 == 0:
@@ -799,8 +788,8 @@ class Visual(Stimulus):
                 ox += 1
             if stimulus.surface_size[1] % 2 == 0:
                 oy += 1
-            stimrect.right = stimrect.right + 1
-            stimrect.bottom = stimrect.bottom + 1
+            stimrect.right += 1
+            stimrect.bottom += 1
             stimrect.center = (ox, oy)
             if selfrect.colliderect(stimrect):
                 return True, None
@@ -842,7 +831,7 @@ class Visual(Stimulus):
             screen_size = _internals.active_exp.screen.surface.get_size()
             self_size = self.surface_size
             if use_absolute_position:
-                self_pos = geometry.position2coordinates(
+                self_pos = geometry.position_to_coordinates(
                     self.absolute_position, screen_size)
                 self_pos[0] -= self_size[0] // 2
                 self_pos[1] -= self_size[1] // 2
@@ -852,7 +841,7 @@ class Visual(Stimulus):
                     self_pos[1] += 1
 
             else:
-                self_pos = geometry.position2coordinates(
+                self_pos = geometry.position_to_coordinates(
                     self.position, screen_size)
                 self_pos[0] -= self_size[0] // 2
                 self_pos[1] -= self_size[1] // 2
@@ -861,7 +850,7 @@ class Visual(Stimulus):
                 if self_size[1] % 2 == 0:
                     self_pos[1] += 1
 
-            pos = geometry.position2coordinates(position, screen_size)
+            pos = geometry.position_to_coordinates(position, screen_size)
             offset = (int(pos[0] - self_pos[0]), int(pos[1] - self_pos[1]))
             self_mask = pygame.mask.from_surface(self._get_surface())
             overlap = False
@@ -876,10 +865,10 @@ class Visual(Stimulus):
         elif mode == "surface":
             screen_size = _internals.active_exp.screen.surface.get_size()
             if use_absolute_position:
-                sx, sy = geometry.coordinates2position(self.absolute_position,
+                sx, sy = geometry.coordinates_to_position(self.absolute_position,
                                                        screen_size)
             else:
-                sx, sy = geometry.coordinates2position(self.position,
+                sx, sy = geometry.coordinates_to_position(self.position,
                                                        screen_size)
             selfrect = pygame.Rect((0, 0), self.surface_size)
             if self.surface_size[0] % 2 == 0:
@@ -887,14 +876,8 @@ class Visual(Stimulus):
             if self.surface_size[1] % 2 == 0:
                 sy += 1
             selfrect.center = (sx, sy)
-            p = geometry.coordinates2position(position, screen_size)
-            p = (position[0] + screen_size[0] // 2,
-                 position[1] + screen_size[1] // 2)
-            p = geometry.coordinates2position(position, screen_size)
-            if selfrect.collidepoint(p):
-                return True
-            else:
-                return False
+            p = geometry.coordinates_to_position(position, screen_size)
+            return selfrect.collidepoint(p)
 
     def plot(self, stimulus):
         """Plot the stimulus on the surface of another stimulus.
@@ -929,7 +912,7 @@ class Visual(Stimulus):
         surface = self._get_surface()
         surface_size = surface.get_size()
         rect = pygame.Rect((0, 0), surface_size)
-        x, y = geometry.position2coordinates(self.position,
+        x, y = geometry.position_to_coordinates(self.position,
                                              stimulus.surface_size)
         if surface_size[0] % 2 == 0:
             x += 1
@@ -976,10 +959,10 @@ class Visual(Stimulus):
         """"Compress the stimulus.
 
         This will create a temporary file on the disk where the surface of the
-        stimululs is written to.
+        stimulus is written to.
         The surface will now be read from the disk to free memory.
         Compressed stimuli cannot do surface operations!
-        Preloading comressed stimuli is possible and highly recommended.
+        Preloading compressed stimuli is possible and highly recommended.
         Depending on the size of the stimulus, this method may take some time
         to compute!
 
@@ -994,7 +977,7 @@ class Visual(Stimulus):
         if self.is_compressed is False:
             if self._compression_filename is None:
                 fid, self._compression_filename = tempfile.mkstemp(
-                    dir=defaults.tempdir, suffix=".tga")
+                    dir=defaults.tempdir, suffix=".bmp")
                 os.close(fid)
             pygame.image.save(self._get_surface(), self._compression_filename)
             self._is_compressed = True
@@ -1065,13 +1048,13 @@ class Visual(Stimulus):
         """
 
         start = get_time()
-        if not _internals.active_exp.is_initialized:
+        if not _internals.active_exp.is_initialised:
             message = "Can't preload stimulus. Expyriment needs to be " + \
-                      "initialized before preloading a stimulus."
+                      "initialised before preloading a stimulus."
             raise RuntimeError(message)
         self._was_compressed_before_preload = self.is_compressed
         if not self.is_preloaded:
-            if _internals.active_exp.screen.open_gl:
+            if _internals.active_exp.screen.opengl:
                 self._ogl_screen = _LaminaPanelSurface(
                     self._get_surface(),
                     position=self.position)
@@ -1118,7 +1101,7 @@ class Visual(Stimulus):
         """
 
         start = get_time()
-        if _internals.active_exp.screen.open_gl:
+        if _internals.active_exp.screen.opengl:
             self._ogl_screen = None
             if self.is_preloaded and not self._was_compressed_before_preload \
                 and keep_surface:
@@ -1173,7 +1156,7 @@ class Visual(Stimulus):
 
         """
 
-        if not _internals.active_exp.is_initialized or\
+        if not _internals.active_exp.is_initialised or\
                              _internals.active_exp.screen is None:
             raise RuntimeError("Cannot not find a screen!")
 
@@ -1187,19 +1170,20 @@ class Visual(Stimulus):
             keep_surface = self.has_surface
             self.preload(inhibit_ogl_compress=True)
 
-        if _internals.active_exp.screen.open_gl:
+        if _internals.active_exp.screen.opengl:
             self._ogl_screen.display()
         else:
             screen = _internals.active_exp.screen.surface
             rect = pygame.Rect((0, 0), self.surface_size)
             screen_size = screen.get_size()
-            x, y = geometry.position2coordinates(self.position, screen_size)
+            x, y = geometry.position_to_coordinates(self.position, screen_size)
             if self.surface_size[0] % 2 == 0:
                 x += 1
             if self.surface_size[1] % 2 == 0:
                 y += 1
             rect.center = (x, y)
             screen.blit(self._get_surface(), rect)
+
         if self._logging:
             _internals.active_exp._event_file_log("Stimulus,drawn,{0}"\
                                    .format(self.id), 2,
@@ -1235,9 +1219,9 @@ class Visual(Stimulus):
         if len(parts) > 1:
             parts[-1] = parts[-1].lower()
         else:
-            parts.append("tga")
+            parts.append("bmp")
         filename = ".".join(parts)
-        pygame.image.save(self._get_surface(), unicode2byte(filename))
+        pygame.image.save(self._get_surface(), unicode_to_bytes(filename))
 
     def picture(self):
         """Return the stimulus as Picture stimulus.
@@ -1254,7 +1238,7 @@ class Visual(Stimulus):
 
         from . import _picture
         fid, location = tempfile.mkstemp(dir=defaults.tempdir,
-                                         suffix=".tga")
+                                         suffix=".bmp")
         os.close(fid)
         pygame.image.save(self._get_surface(), location)
         return _picture.Picture(filename=location)
@@ -1346,8 +1330,8 @@ class Visual(Stimulus):
             factors[1] = abs(factors[1])
         self._set_surface(pygame.transform.smoothscale(
             self._get_surface(),
-            (int(misc.round(self.surface_size[0] * factors[0])),
-             int(misc.round(self.surface_size[1] * factors[1])))))
+            (int(misc.py2_round(self.surface_size[0] * factors[0])),
+             int(misc.py2_round(self.surface_size[1] * factors[1])))))
         if True in flip:
             self.flip(flip)
         if self._logging:
@@ -1436,7 +1420,7 @@ class Visual(Stimulus):
         Parameters
         ----------
         level : int
-            level of bluring
+            level of blurring
 
         Returns
         -------
@@ -1455,7 +1439,7 @@ class Visual(Stimulus):
         self.scale((level, level))
         if self._logging:
             _internals.active_exp._event_file_log(
-                "Stimulus,blured,{0}, level={1}".format(self.id, level), 2)
+                "Stimulus,blurred,{0}, level={1}".format(self.id, level), 2)
         return int((get_time() - start) * 1000)
 
     def scramble(self, grain_size):
