@@ -1,18 +1,14 @@
-"""
-The stimuli extra package.
+"""The stimuli extra package.
 
 Notes
 -----
-    To us the extras module you have to import it manually by calling:
-    `import expyriment.stimuli.extras`
+To us the extras module you have to import it manually by calling:
+`import expyriment.stimuli.extras`
 
 """
 
 __author__ = 'Florian Krause <florian@expyriment.org> \
 Oliver Lindemann <oliver@expyriment.org>'
-__version__ = ''
-__revision__ = ''
-__date__ = ''
 
 from ... import _internals
 
@@ -27,5 +23,5 @@ for name, code in _internals.import_plugins_code("stimuli").items():
 
 try:
     del (name, code)
-except:
+except Exception:
     pass
